from django.core.management.base import BaseCommand
from api.models import AcceptedPass, CvImageSensor, EntranceExit, ParkingLot, Pass, User
from . import _lot_data_parser, _geojson_parser, _enex_loc_parser

# to run: python manage.py seed --mode=refresh

""" Clear all data then re-create it"""
MODE_REFRESH = "refresh"

""" Clear all data and do not create any objects """
MODE_CLEAR = "clear"


class Command(BaseCommand):
    help = "seed database for testing and development."

    def add_arguments(self, parser):
        parser.add_argument("--mode", type=str, help="Mode")
        parser.add_argument("--amount", type=int, help="Amount of ParkingLots to create")

    def handle(self, *args, **options):
        self.stdout.write("seeding data...")
        run_seed(self, options["mode"], options["amount"])
        self.stdout.write("done.")


def clear_data():
    """Deletes all the table data"""
    AcceptedPass.objects.all().delete()
    CvImageSensor.objects.all().delete()
    EntranceExit.objects.all().delete()
    ParkingLot.objects.all().delete()
    Pass.objects.all().delete()
    User.objects.filter(is_superuser=False).delete() #delete all api_user except superuser


def create_lots_passes_and_accepted_passes(amount):
    """Fill the database with ParkingLot, EntranceExit, Pass and AcceptedPass objects"""
    # create passes (U, A, and D)
    u_pass = Pass(passid=1, shortname="U", name="U Pass")
    a_pass = Pass(passid=2, shortname="A", name="A Pass")
    d_pass = Pass(passid=3, shortname="D", name="Disabled Spots")
    v_pass = Pass(passid=4, shortname="V", name="Visitor Parking")
    p_pass = Pass(passid=5, shortname="P", name="Paid Parking")
    u_pass.save()
    d_pass.save()
    a_pass.save()
    v_pass.save()
    p_pass.save()

    # load in lot data 
    lot_number_to_name_and_passes = _lot_data_parser.get_lots_and_passes()
    lot_number_to_loc = _geojson_parser.get_lot_locations()
    lot_number_to_enex_locs = _enex_loc_parser.get_enex_locations()

    # create db models on a lot-by-lot basis
    i = 0
    for num in lot_number_to_name_and_passes:
        if i == amount:
            break
        

        if num in lot_number_to_loc:
            lat = lot_number_to_loc[num][0]
            lon = lot_number_to_loc[num][1]
        elif int(num) in lot_number_to_loc:
            lat = lot_number_to_loc[int(num)][0]
            lon = lot_number_to_loc[int(num)][1]
        else:
            continue

        # create new lot
        new_lot = ParkingLot(
            lotid=num,
            name=lot_number_to_name_and_passes[num][0],
            usedstalls=0,
            latitude=lat,
            longitude=lon,
        )
        new_lot.save()
        
        if num in lot_number_to_enex_locs:
            for loc in lot_number_to_enex_locs[num]:
                create_enex_and_sensor(num, new_lot, loc)
        else:
            create_enex_and_sensor(num, new_lot, (lat, lon))


        # create accepted passes for the lot
        for j, accpass in enumerate(lot_number_to_name_and_passes[num][1]):
            if accpass != None and accpass != 0:
                new_acc_pass = AcceptedPass(
                    lotid=ParkingLot.objects.filter(pk=num)[0],
                    passid=Pass.objects.filter(pk=j+1)[0],
                    allottedstalls=accpass,
                )
                new_acc_pass.save()
        i += 1



def run_seed(self, mode, amt):
    """Seed database based on mode

    :param mode: refresh / clear
    :return:
    """
    # clear data and create new rows
    if mode == MODE_REFRESH:
        clear_data()
        create_lots_passes_and_accepted_passes(amt)
        return
    # only clear data
    if mode == MODE_CLEAR:
        clear_data()
        return
    # add new rows without clearing old 
    create_lots_passes_and_accepted_passes(amt)


def create_enex_and_sensor(lotid, lot, enex_loc):
    # create new EntranceExit for the lot
    new_enex = EntranceExit(
        lotid=lot,
        latitude=enex_loc[0],
        longitude=enex_loc[1],
    )
    new_enex.save()

    # create new sensor object for the EntranceExit
    # first create a new sensor user
    new_sensor_user = User.objects.create_user(
                            username=f'sensor{new_enex.enid}',
                            password=f'sensor{new_enex.enid}',
                            is_sensor=True)
    new_sensor_user.save()
    
    # then create the actual sensor
    new_sensor = CvImageSensor(
        user=new_sensor_user,
        enid=new_enex,
        timelastupdate=None,
        isentranceside=1,
        imageurl=None,
        imagetime=None
    )
    new_sensor.save()
