# Imports for testing
from datetime import datetime
from django.urls import reverse
from library.predictor import UpdateType
from django.conf import settings
from rest_framework.test import APITestCase, APIClient
from .models import AcceptedPass, ParkingLot, CvImageSensor, Pass, User, EntranceExit
from django.test import RequestFactory
from django.test.runner import DiscoverRunner
from rest_framework.permissions import IsAdminUser
from numbers import Number #quick check if an object is a number
import pytz #for timezone info
import library.predictor as predictor
from api.views import IsSensorUser, AllowReadOnlyMethods
from django.test import TestCase
import pandas as pd
import torch
import filecmp
from django.core.files.uploadedfile import SimpleUploadedFile


class ExistingDBTestRunner(DiscoverRunner):

    def setup_databases(self, **kwargs):
        pass

    def teardown_databases(self, old_config, **kwargs):
        pass


#---------------------Permission TEST-----------------------#
class PermissionTests(APITestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.sensor_user = User.objects.create(username='sensor1', 
                                               is_staff=0, 
                                               is_sensor=1, 
                                               is_active=1, 
                                               is_superuser=0)

        self.admin_user = User.objects.create(username='admin1', 
                                               is_staff=1, 
                                               is_sensor=0, 
                                               is_active=1, 
                                               is_superuser=1)

        self.normal_user = User.objects.create(username='norm1', 
                                               is_staff=0, 
                                               is_sensor=0, 
                                               is_active=0, 
                                               is_superuser=0)

    def test_IsAdminUser(self):
        request = self.factory.get('/')
        request.user = self.admin_user
        permission = IsAdminUser().has_permission(request, None)
        self.assertTrue(permission)

    def test_IsSensorUser(self):
        request = self.factory.get('/')
        request.user = self.sensor_user
        permission = IsSensorUser().has_permission(request, None)
        self.assertTrue(permission)

        request = self.factory.patch('/')
        request.user = self.admin_user
        permission = IsSensorUser().has_permission(request, None)
        self.assertFalse(permission)

        request = self.factory.patch('/')
        request.user = self.normal_user
        permission = IsSensorUser().has_permission(request, None)
        self.assertFalse(permission)

    def test_AllowReadOnlyMethods(self):
        request = self.factory.patch('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertFalse(permission)

        request = self.factory.post('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertFalse(permission)

        request = self.factory.delete('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertFalse(permission)

        request = self.factory.put('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertFalse(permission)

        request = self.factory.get('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertTrue(permission)

        request = self.factory.head('/')
        request.user = self.admin_user
        permission = AllowReadOnlyMethods().has_permission(request, None)
        self.assertTrue(permission)


#---------------------AUTHENTICATION TEST-----------------------#
class AuthenticationTests(APITestCase):
    def setUp(self):
        self.su_tester_creds={"username": "tester1", "password":"tester1"}
        self.su_tester1 = User.objects.create_superuser(**self.su_tester_creds, is_superuser=True)
        authenticate_url = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_url, data=self.su_tester_creds)
        self.assertEqual(response_token.status_code, 200)
        self.su_token = response_token.json()['access']

        self.lotdata1 = {
            'lotid' : 1,
            'name' : 'Test_Case_Lot_1',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
        }
        lot1 = ParkingLot.objects.create(**self.lotdata1)

        self.passdata1 = {
            'passid' : 1,
            'shortname' : 'U',
            'name' : 'Test_Pass_1'
        }
        pass1 = Pass.objects.create(**self.passdata1)

        self.passdata2 = {
            'passid' : 2,
            'shortname' : 'A',
            'name' : 'Test_Pass_2'
        }
        pass2 = Pass.objects.create(**self.passdata2)

        self.accpassdata1 = {
            'accpassid':1,
            'lotid' : lot1,
            'passid' : pass1,
            'allottedstalls' : 100
        }
        accpass1 = AcceptedPass.objects.create(**self.accpassdata1)

        self.enexdata1 = {
            'enid' : 1,
            'lotid' : lot1,
            'longitude' : 1.2,
            'latitude' : 3.2
        }
        enex1 = EntranceExit.objects.create(**self.enexdata1)

        self.enexdata2 = {
            'enid' : 2,
            'lotid' : lot1,
            'longitude' : 1.2,
            'latitude' : 3.2
        }
        enex2 = EntranceExit.objects.create(**self.enexdata2)

        self.sensor_tester_creds={'username':'sensor1', 'password':'sensor1'}
        authenticate_url = reverse('cookie_token_obtain_pair')
        self.sensor_tester1 = User.objects.create_user(**self.sensor_tester_creds,  
                              is_sensor=True, 
                              is_staff=True, 
                              is_superuser=False)
        response_token = self.client.post(authenticate_url, data=self.sensor_tester_creds)
        self.sensor_token = response_token.json()['access']

        self.sensordata1 = {
            'user': self.sensor_tester1,
            'sensorid':1,
            'enid':enex1,
            'timelastupdate':datetime.fromtimestamp(0, tz=pytz.utc),
            'isentranceside':1,
            'imageurl':'',
            'imagetime':datetime.fromtimestamp(0, tz=pytz.utc)
        }
        CvImageSensor.objects.create(**self.sensordata1)

        self.admin_tester_creds={"username":"tester2", "password":"tester2"}
        self.admin_tester1 = User.objects.create_superuser(**self.admin_tester_creds,  
                              is_sensor=False, 
                              is_staff=True, 
                              is_superuser=True)
        response_token = self.client.post(authenticate_url, data=self.admin_tester_creds)
        self.admin_token = response_token.json()['access']

        # self.admin_allowed_views = [views.CountParkingLotView,
        #                        views.DetailParkingLotView,
        #                        views.ListParkingLotsView,
        #                        views.ListLotSensorsView,
        #                        views.ParkingLotPercentFullView,
        #                        views.ListParkingLotPercentFullView,
        #                        views.ListLotAcceptedPassesView,
        #                        views.ListLotAvailableAcceptedPassesView,
        #                        views.ListLotEnExesView,
        #                        views.DetailEnExView,
        #                        views.ListEnExView,
        #                        views.DetailPassView,
        #                        views.ListPassesView,
        #                        views.DetailAcceptedPassView,
        #                        views.ListAcceptedPassesView,
        #                        views.DetailSensorView,
        #                        views.ListSensorsView,
        #                        views.MediaStorageView,
        #                        views.InitSystemInfoView,
        #                        views.PredictionHalfHourView,
        #                        views.PredictionWeekView,
        #                        views.ParkingLotCapacityView,
        #                        views.DummyView,
        #                        views.LotSearchView,
        #                        views.BuildingSearchView]


    def test_post_token(self):
        """
        Testing if a created user from "setUp" can authenticate
        """
        authenticate_url = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_url, data=self.su_tester_creds)
        self.assertEqual(response_token.status_code, 200)

    def test_get_bad_token(self):
        """
        Testing if I get a bad cookie with the token/blacklistme/ endpoint
        """
        badtoken_url = reverse('cookie_bad_token')
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.su_token}"}
        response_token = self.client.get(badtoken_url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_count_parking_lot_view(self):
        url = reverse('lotcount-pk', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.patch(url, data={'usedstalls':10}, **arguments) 
        self.assertEqual(response_token.status_code, 200)
        
    def test_admin_detail_parking_lot_view(self):
        url = reverse('lot-pk', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.patch(url, data=self.lotdata1, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.delete(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_parking_lots_view(self):
        url = reverse('lot-list')
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        new_lot_data = {
            'lotid' : 2,
            'name' : 'Test_Case_Lot_2',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
        }
        response_token = self.client.post(url, data=new_lot_data, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_lot_sensors_view(self):
        url = reverse('lot-sensor-list', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_parking_lot_percent_full_view(self):
        url = reverse('lot-percent-full', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_parking_lot_percent_full_view(self):
        url = reverse('list-lots-percent-full')
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_lot_accepted_passes_view(self):
        url = reverse('lot-accpass-list', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_lot_available_accepted_passes_view(self):
        url = reverse('lot-available-accpasstoadd-list', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_lot_en_exes_view(self):
        url = reverse('lot-enex-list', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_detail_en_ex_view(self):
        url = reverse('single-enex', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        new_enex = {
            'enid' : 1,
            'lotid' : 1,
            'longitude' : 1.2,
            'latitude' : 3.2
        }
        response_token = self.client.patch(url, data=new_enex, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.delete(url, **arguments)
        self.assertEqual(response_token.status_code, 200)


    def test_admin_list_en_ex_view(self):
        url = reverse('enex-create')
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        new_enex = {
            'enid' : 1,
            'lotid' : 1,
            'longitude' : 1.2,
            'latitude' : 3.2
        }
        response_token = self.client.post(url, format='json', data=new_enex, **arguments)
        self.assertEqual(response_token.status_code, 200)

        url = reverse('enex-list')
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)
    
    def test_admin_detail_pass_view(self):
        url = reverse('single-pass', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        new_pass = {'passid':3,
                    'shortname':'E',
                    'name':'Test_Pass_3'}
        response_token = self.client.patch(url, data=new_pass, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.delete(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_passes_view(self):
        url = reverse('pass-create')
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        new_pass = {'shortname':'E',
                    'name':'Test_Pass_3'}
        response_token = self.client.post(url, format='json', data=new_pass, **arguments)
        self.assertEqual(response_token.status_code, 200)

        url = reverse('pass-list')
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_detail_accepted_pass_view(self):
        url = reverse('single-accepted-pass', args=(1,))
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        new_acc_pass = {'lotid':1,
                        'passid':2,
                        'allottedstalls':200}
        response_token = self.client.patch(url, data=new_acc_pass, **arguments)
        self.assertEqual(response_token.status_code, 200)

        response_token = self.client.delete(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_accepted_passes_view(self):
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        url = reverse('accepted-pass-create')
        new_acc_pass = {'lotid':1,
                        'passid':2,
                        'allottedstalls':200}
        response_token = self.client.post(url, format='json', data=new_acc_pass, **arguments)
        self.assertEqual(response_token.status_code, 200)

        url = reverse('accepted-pass-list')
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_detail_sensor_view(self):
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        url = reverse('single-sensor', args=(1,))
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

        new_sensor = {
            'user': self.sensor_tester1.pk,
            'sensorid':1,
            'enid':1,
            'timelastupdate':datetime.fromtimestamp(0),
            'isentranceside':1,
            'imageurl':'',
            'imagetime':datetime.fromtimestamp(0)
        }
        url = reverse('single-sensor', args=(1,))
        response_token = self.client.patch(url, data=new_sensor, **arguments)
        self.assertEqual(response_token.status_code, 200)

        url = reverse('single-sensor', args=(1,))
        response_token = self.client.delete(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_list_sensors_view(self):
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        url = reverse('sensor-create')
        new_sensor = {'user': self.sensor_tester1.pk,
            'sensorid':2,
            'enid':2,
            'timelastupdate':datetime.fromtimestamp(0),
            'isentranceside':1,
            'imageurl':'',
            'imagetime':datetime.fromtimestamp(0)}
        response_token = self.client.post(url, format='json', data=new_sensor, **arguments)
        self.assertEqual(response_token.status_code, 200)

        url = reverse('sensor-list')
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)

    def test_admin_media_storage_view(self):
        arguments= {"HTTP_AUTHORIZATION" :f"Bearer {self.admin_token}"}

        url = reverse('media-storage')
        response_token = self.client.get(url, **arguments)
        self.assertEqual(response_token.status_code, 200)
        
        new_media = {'user': self.sensor_tester1.pk,
            'description':'Test_Media1',
            'file':SimpleUploadedFile('media/images/profile.png', b'file_content', content_type='image/png'),
            'folder':'media/images/'}
        response_token = self.client.post(url, data=new_media, **arguments)
        self.assertEqual(response_token.status_code, 200)

#---------------------PARKING LOT TESTING-----------------------#
class ParkingLotTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(ParkingLotTests,self).setUpClass()
        self.lotdata1 = {
            'lotid' : 1,
            'name' : 'Test_Case_Lot_1',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
        }
        self.lot1 = ParkingLot(**self.lotdata1)
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access']     

    def test_get_parkinglot(self):
        """
        Trying to get a list of all parking lots for a 200 status code
        """
        url = reverse('lot-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_get_parkinglot_notfound(self):
        """
        Trying to find a non existing parking lot will give you a 404 status code
        """
        url = reverse('lot-pk', args=(999999,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_post_notauthenticated(self):
        """
        A non-authenticated user should not be allowed to do a post request to parkinglot views/endpoints
        """
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1)
        self.assertEqual(response.status_code, 401) #401 = unauthorized - no credentials provided

        # With a wrong token
        response = self.client.post(view, data=self.lotdata1, **{"HTTP_AUTHORIZATION" :f"NOTGONNAWORKSORRYMR.DUDE"})
        self.assertEqual(response.status_code, 401) # 401 (or403 = forbidden )

    def test_post_authenticated_newlot(self):
        """
        A test that use the admin's credentials to make a new parking lot
        """
        # creating/POST a new lot successfully?
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        # Can I get that lot?
        url = reverse('lot-pk', args=(self.lotdata1['lotid'],))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        ## That lot has equal fields to what i created?
        json_data = response.json()
        for key in json_data:
            if key in self.lotdata1: ##Some fields like timecount update didn't need to be declared
                if (isinstance(self.lotdata1[key],Number) or isinstance(json_data[key],Number)):
                    self.assertAlmostEqual(float(self.lotdata1[key]), float(json_data[key]), 5)
                else:
                    self.assertEqual(self.lotdata1[key], json_data[key])
        
        url = reverse('lot-pk', args=(self.lotdata1['lotid'],))
        response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
        self.assertEqual(response.status_code, 200)

    def test_post_auth_baddata(self):
        """
        A post to create (authenticated ofc) a new lot but with bad data will error
        """
        # creating/POST a new lost with bad lotid
        badid = {
            'lotid' : "sixninefourtwoone",
            'name' : 'fidsjf',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
            # 'timecountupdated' : None,
        }
        view = reverse('lot-list')
        response = self.client.post(view, data=badid, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 400) #400 = Bad Request
        # creating/POST a new lost with bad coordinates
        badcoordinates1 = {
            'lotid' : 49290,
            'name' : 'okay',
            'latitude' : "notgonnawurk",
            'longitude' : 10,
            'usedstalls' : 29,
            # 'timecountupdated' : None,
        }
        response = self.client.post(view, data=badcoordinates1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 400) #400 = Bad Request
        #creating/POST a new lost with bad coordinates (too many decimals)
        badcoordinates2 = {
            'lotid' : 49290,
            'name' : 'okay2',
            'latitude' : 202,
            'longitude' : 10.0000000001,
            'usedstalls' : 29,
            # 'timecountupdated' : None,
        }
        response = self.client.post(view, data=badcoordinates2, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 400) #400 = Bad Request
        #creating/POST a new lost with bad coordinates (too many decimals)
        badcoordinates3 = {
            'lotid' : 49290,
            'name' : 'okay2',
            'latitude' : 202,
            'longitude' : 10.001,
            'usedstalls' : "areyouokayannie",
            'timecountupdated' : datetime.now(pytz.utc),
        }
        response = self.client.post(view, data=badcoordinates3, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 400) #400 = Bad Request

    def test_lotpost_stresscreate(self):
        """
        Creating many random parkinglot and check if they were all added
        """
        view = reverse('lot-list')
        rand_numbers = list(range(100,200))
        for num in rand_numbers:
            newdata={
                'lotid' : num,
                'name' : str(num),
                'latitude' : num+0.001,
                'longitude' : num+0.0005,
                'usedstalls' : num+1,
                'timecountupdated' : datetime.now(pytz.utc),
            }
            response = self.client.post(view, data=newdata, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created
            
            # Delete the lots for future tests in this class
            url = reverse('lot-pk', args=(num,))
            response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
            self.assertEqual(response.status_code, 200)

    # TODO: Test ListLotSensorsView new endpoint 
#---------------------ENTRANCE/EXIT TESTING-----------------------#
class EntranceExitTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(EntranceExitTests,self).setUpClass()
        self.lotdata1 = {
            'lotid' : 1,
            'name' : 'Test_Case_Lot_1',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
            'timecountupdated' :  datetime.now(pytz.utc),
        }
        self.lot1 = ParkingLot(**self.lotdata1)

        self.enexdata1 = {
            'lotid' : 1,
            'longitude' : 10,
            'latitude' : 221,           
        }
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access']     
    
    def test_get_entranceexit(self):
        """
        Trying to get a list of all entrance/exits for a 200 status code
        """
        url = reverse('enex-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
    
    def test_get_listlotentranceexitsview(self):
        """
        Trying to get a list of all entrance/exits in a specific lot for a 200 status code
        """
        url = reverse('lot-enex-list', args=(1,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_get_enex_notfound(self):
        """
        Trying to find a non existing entrance/exit will give you a 404 status code
        """
        url = reverse('single-enex', args=(999999,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
    
    def test_post_notauthenticated(self):
        """
        A non-authenticated user should not be allowed to do a post request to enex views/endpoints
        """
        # Post a parking lot to assign the entrance/exit to
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1,**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting a lot should work 

        # Attempt to assign the entrance exit without credentials
        view = reverse('enex-list')
        response = self.client.post(view, data=self.enexdata1)
        self.assertEqual(response.status_code, 401) #401 = unauthorized - no credentials provided

        # Try also with a wrong token
        response = self.client.post(view, data=self.enexdata1, **{"HTTP_AUTHORIZATION" :f"NOTGONNAWORKSORRYMR.DUDE"})
        self.assertEqual(response.status_code, 401) # 401 (or403 = forbidden )
    
    def test_post_authenticated_newentrance(self):
        """
        A test that use the admin's credentials to make a new entrance/exit
        """
        # Create lot to associate with entrance
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        
        # Creating (POST) a new entrance successfully
        view = reverse('enex-create')
        response = self.client.post(view, data=self.enexdata1, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        # Can a GET see the entrance that was just made
        enid = EntranceExit.objects.order_by('-enid')[0].enid
        url = reverse('single-enex', args=(enid,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Does the entrance has equal fields to what was created
        json_data = response.json()
        for key in json_data:
            if key in self.enexdata1: 
                if (isinstance(self.enexdata1[key],Number) or isinstance(json_data[key],Number)):
                    self.assertAlmostEqual(float(self.enexdata1[key]), float(json_data[key]), 5)
                else:
                    self.assertEqual(self.enexdata1[key], json_data[key])        

        # Delete the lot for future tests in this class
        url = reverse('lot-pk', args=(self.lotdata1['lotid'],))
        response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
        self.assertEqual(response.status_code, 200)

    def test_entrancepost_stresscreate(self):
        """
        Creating many random entrances and check if they were all added
        """
        view = reverse('lot-list')
        for num in range(1,100):
            newdata={
                'lotid' : num,
                'name' : str(num),
                'latitude' : num+0.001,
                'longitude' : num+0.0005,
                'usedstalls' : num+1,
                'timecountupdated' : datetime.now(pytz.utc),
            }
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created

        # Add entrances to each parking lot
        for num in range (1,100):        
            newenexdata={
                'lotid' : num,                
                'latitude' : num+0.001,
                'longitude' : num+0.0005,               
            }

            # Creating (POST) a new entrance successfully
            view = reverse('enex-create')
            response = self.client.post(view, data=newenexdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200)
            
            # Delete the lots for future tests in this class
            url = reverse('lot-pk', args=(num,))
            response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
            self.assertEqual(response.status_code, 200)

#---------------------PASS TESTING-----------------------#
class PassTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(PassTests,self).setUpClass()
        self.passdata1 = {
            'shortname' : 'U',
            'name' : 'University of Utah Test Parking Pass'
        }
        self.pass1 = Pass(**self.passdata1)
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access'] 

    def test_get_pass(self):
        """
        Trying to get a list of all passes for a 200 status code
        """
        url = reverse('pass-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
    
    def test_get_pass_notfound(self):
        """
        Trying to find a non existing pass will give you a 404 status code
        """
        url = reverse('single-pass', args=(999999,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
    
    def test_post_notauthenticated(self):
        """
        A non-authenticated user should not be allowed to do a post request to pass views/endpoints
        """ 
        # Attempt to assign the pass without credentials
        view = reverse('pass-list')
        response = self.client.post(view, data=self.passdata1)
        self.assertEqual(response.status_code, 401) #401 = unauthorized - no credentials provided

        # Try also with a wrong token
        response = self.client.post(view, data=self.passdata1, **{"HTTP_AUTHORIZATION" :f"NOTGONNAWORKSORRYMR.DUDE"})
        self.assertEqual(response.status_code, 401) # 401 (or 403 = forbidden ) 

    def test_post_authenticated_pass(self):
        """
        A test that use the admin's credentials to make a new pass
        """    
        # Creating/POST a new pass successfully
        view = reverse('pass-create')
        response = self.client.post(view, data=self.passdata1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        # Can I get that pass?
        passid = Pass.objects.order_by('-passid')[0].passid
        url = reverse('single-pass', args=(passid,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # That pass has equal fields to what i created?
        json_data = response.json()
        for key in json_data:
            if key in self.passdata1: 
                if (isinstance(self.passdata1[key],Number) or isinstance(json_data[key],Number)):
                    self.assertAlmostEqual(float(self.passdata1[key]), float(json_data[key]), 5)
                else:
                    self.assertEqual(self.passdata1[key], json_data[key])
        
        url = reverse('single-pass', args=(passid,))
        response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
        self.assertEqual(response.status_code, 200)

    def test_pass_stresscreate(self):
        """
        Creating many random passes and check if they were all added
        """
        view = reverse('pass-create')
        for num in range(1,99):
            newdata={                
                'shortname' : str(num),
                'name' : str(num)               
            }
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            
            self.assertEqual(response.status_code, 200) #200=good, created        
            
            # Delete the passes for future tests in this class
            url = reverse('single-pass', args=(num,))
            response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
            self.assertEqual(response.status_code, 200)  

#---------------------ACCEPTED PASS TESTING-----------------------#
class AccPassTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(AccPassTests,self).setUpClass()
        self.passdata1 = {
            'shortname' : 'U',
            'name' : 'University of Utah Test Parking Pass'
        }
        self.pass1 = Pass(**self.passdata1)  

        self.lotdata1 = {
            'lotid' : 1,
            'name' : 'Test_Case_Lot_1',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
            'timecountupdated' :  datetime.now(pytz.utc),
        }
        self.lot1 = ParkingLot(**self.lotdata1)    

        self.accpassdata1 = {
            'lotid' : 1,
            'passid' : 1,
            'allottedstalls' : 100,           
        }
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access'] 

    def test_get_accpass(self):
        """
        Trying to get a list of all acc passes for a 200 status code
        """
        url = reverse('accepted-pass-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
    
    def test_get_listlotacceptedpassesview(self):
        """
        Trying to get a list of all accepted passes in a specific lot for a 200 status code
        """
        url = reverse('lot-accpass-list', args=(1,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_get_accpass_notfound(self):
        """
        Trying to find a non existing acc pass will give you a 404 status code
        """
        url = reverse('single-accepted-pass', args=(999999,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
    
    def test_post_notauthenticated(self):
        """
        A non-authenticated user should not be allowed to do a post request to acc pass views/endpoints
        """
        # Post a lot to assign the accepted pass to
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1,**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting a lot should work 

        # Post a pass to assign the accepted pass to
        view = reverse('pass-list')
        response = self.client.post(view, data=self.passdata1,**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting a lot should work

        # Update accepted pass to correspond to newly created pass
        self.accpassdata1 = {
        'lotid' : 1,
        'passid' : Pass.objects.order_by('-passid')[0].passid,
        'allottedstalls' : 100,           
        }

        # Attempt to assign the accepted pass without credentials
        view = reverse('accepted-pass-list')
        response = self.client.post(view, data=self.accpassdata1)
        self.assertEqual(response.status_code, 401) #401 = unauthorized - no credentials provided

        # Try also with a wrong token
        response = self.client.post(view, data=self.accpassdata1, **{"HTTP_AUTHORIZATION" :f"NOTGONNAWORKSORRYMR.DUDE"})
        self.assertEqual(response.status_code, 401) # 401 (or403 = forbidden )

    def test_post_authenticated_accpass(self):
        """
        A test that use the admin's credentials to make a new accepted pass
        """
        # Create lot to associate with acc pass
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)

        # Post a pass to assign the accepted pass to
        view = reverse('pass-list')
        response = self.client.post(view, data=self.passdata1,**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting a lot should work 
        
        # Update acc pass with pass data
        self.accpassdata1 = {
        'lotid' : 1,
        'passid' : Pass.objects.order_by('-passid')[0].passid,
        'allottedstalls' : 100,           
        }

        # Creating (POST) a new acc pass successfully
        view = reverse('accepted-pass-create')
        response = self.client.post(view, data=self.accpassdata1, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})

        self.assertEqual(response.status_code, 200)
        # Can a GET see the acc pass that was just made
        accpassid = AcceptedPass.objects.order_by('-accpassid')[0].accpassid
        url = reverse('single-accepted-pass', args=(accpassid,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Does the accepted pass have equal fields to what was created
        json_data = response.json()
        for key in json_data:
            if key in self.accpassdata1: 
                if (isinstance(self.accpassdata1[key],Number) or isinstance(json_data[key],Number)):
                    self.assertAlmostEqual(float(self.accpassdata1[key]), float(json_data[key]), 5)
                else:
                    self.assertEqual(self.accpassdata1[key], json_data[key])        

        # Delete the accepted pass for future tests in this class
        url = reverse('single-accepted-pass', args=(accpassid,))
        response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
        self.assertEqual(response.status_code, 200)

    def test_accpasspost_stresscreate(self):
        """
        Creating many random lots and check if they were all added
        """
        view = reverse('lot-list')
        for num in range(1,10):
            newdata={
                'lotid' : num,
                'name' : str(num),
                'latitude' : num+0.001,
                'longitude' : num+0.0005,
                'usedstalls' : num+1,
                'timecountupdated' : datetime.now(pytz.utc),
            }
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created
        
        view = reverse('pass-list')
        for num in range(1,10):
            newdata={         
            'shortname' : 'U' + str(num),
            'name' : 'University of Utah Test Parking Pass' + str(num)
            }            
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created

        # Add accepted passes to each parking lot
        for num in range (1,10):        
            newaccpassdata={
                'lotid' : num,                
                'passid' : Pass.objects.order_by('-passid')[0].passid,
                'allottedstalls' : num,               
            }            
         
            # Creating (POST) a new accepted pass successfully
            view = reverse('accepted-pass-create')
            response = self.client.post(view, data=newaccpassdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200)
            
            # Delete the passes for future tests in this class
            url = reverse('single-accepted-pass', args=(num,))
            response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
            self.assertEqual(response.status_code, 200)

#---------------------SENSOR TESTING-----------------------#
class SensorTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(SensorTests,self).setUpClass()      

        self.lotdata1 = {
            'lotid' : 1,
            'name' : 'Test_Case_Lot_1',
            'latitude' : 221,
            'longitude' : 10,
            'usedstalls' : 29,
            'timecountupdated' :  datetime.now(pytz.utc),
        }
        self.lot1 = ParkingLot(**self.lotdata1)    

        self.enexdata1 = {
            'lotid' : 1,
            'longitude' : 10,
            'latitude' : 221,           
        }
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access'] 
    
    def test_get_sensor(self):
        """
        Trying to get a list of all sensors for a 200 status code
        """
        url = reverse('sensor-list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
    
    def test_get_listlotsensorsview(self):
        """
        Trying to get a list of all sensors in a specific lot for a 200 status code
        """
        url = reverse('lot-sensor-list', args=(1,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_get_sensor_notfound(self):
        """
        Trying to find a non existing sensor will give you a 404 status code
        """
        url = reverse('single-sensor', args=(999999,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)
    
    def test_post_notauthenticated(self):
        """
        A non-authenticated user should not be allowed to do a post request to sensor views/endpoints
        """
        # Post a lot to assign the enex to
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1,**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting a lot should work 

        # Post a enex to assign the sensor to
        view = reverse('enex-list')
        response = self.client.post(view, data=self.enexdata1,format='json',**{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200) # Posting an enex should work

        self.sensor1 = {
            'enid' : EntranceExit.objects.order_by('-enid')[0].enid,
            'isentranceside' : 1,
        }         

        # Attempt to assign sensor without credentials
        view = reverse('sensor-list')
        response = self.client.post(view, data=self.sensor1)
        self.assertEqual(response.status_code, 401) #401 = unauthorized - no credentials provided

        # Try also with a wrong token
        response = self.client.post(view, data=self.sensor1, **{"HTTP_AUTHORIZATION" :f"NOTGONNAWORKSORRYMR.DUDE"})
        self.assertEqual(response.status_code, 401) # 401 (or403 = forbidden )

    def test_post_authenticated_newsensor(self):
        """
        A test that use the admin's credentials to make a new sensor
        """
        # Create lot to associate with entrance
        view = reverse('lot-list')
        response = self.client.post(view, data=self.lotdata1, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        
        # Creating (POST) a new entrance successfully
        view = reverse('enex-create')
        response = self.client.post(view, data=self.enexdata1, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)

        self.sensor1 = {
            'enid' : EntranceExit.objects.order_by('-enid')[0].enid,
            'isentranceside' : 1,
        }  

        # Create a new sensor successfully
        view = reverse('sensor-create')
        response = self.client.post(view, data=self.sensor1, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)

        # Can a GET see the sensor that was just made
        sensorid = CvImageSensor.objects.order_by('-sensorid')[0].sensorid
        url = reverse('single-sensor', args=(sensorid,))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Does the entrance has equal fields to what was created
        json_data = response.json()
        for key in json_data:
            if key in self.sensor1: 
                if (isinstance(self.sensor1[key],Number) or isinstance(json_data[key],Number)):
                    self.assertAlmostEqual(float(self.sensor1[key]), float(json_data[key]), 5)
                else:
                    self.assertEqual(self.sensor1[key], json_data[key])        

        # Delete the sensor for future tests in this class
        url = reverse('single-sensor', args=(sensorid,))
        response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
        self.assertEqual(response.status_code, 200)
    
    def test_sensorpost_stresscreate(self):
        """
        Creating many random sensors and check if they were all added
        """
        view = reverse('lot-list')
        for num in range(1,10):
            newdata={
                'lotid' : num,
                'name' : str(num),
                'latitude' : num+0.001,
                'longitude' : num+0.0005,
                'usedstalls' : num+1,
                'timecountupdated' : datetime.now(pytz.utc),
            }
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created
        
        view = reverse('enex-list')
        for num in range(1,10):
            newdata = {
            'lotid' : num,
            'longitude' : num+.001,
            'latitude' : num+.005,           
        }          
            response = self.client.post(view, data=newdata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200) #200=good, created

        # Add sensors to each parking lot entrance
        for num in range (1,10):        
            newsensordata={              
            'enid' : EntranceExit.objects.order_by('-enid')[num-1].enid,
            'isentranceside' : 1,                        
            }            
         
            # Creating (POST) a new sensor successfully
            view = reverse('sensor-create')
            response = self.client.post(view, data=newsensordata, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
            self.assertEqual(response.status_code, 200)
            
            # Delete the sensors for future tests in this class
            url = reverse('single-sensor', args=(num,))
            response = self.client.delete(url, **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})    
            self.assertEqual(response.status_code, 200)


#---------------------PREDICTION TESTING-----------------------#
class PredictorTestCase(TestCase):
    # def setUp(self):
    #     self.lot = ParkingLot.objects.create(lotid=99990, name='test', longitude=1, latitude=1, usedstalls=10, timecountupdated=datetime.now(pytz.utc))
    #     self.pas = Pass.objects.create(passid=1234, shortname='ts', name='testing a lot')
    #     self.pas2 = Pass.objects.create(passid=1235, shortname='tt', name='testing a lot2')
    #     AcceptedPass.objects.create(accpassid=99997, lotid=self.lot, passid=self.pas, allottedstalls=123)
    #     AcceptedPass.objects.create(accpassid=99998, lotid=self.lot, passid=self.pas2, allottedstalls=51)

    # def test_get_capacity(self):
    #     self.assertEqual(predictor.get_capacity(99990), 174)
    #     self.pas3 = Pass.objects.create(passid=1236, shortname='tr', name='testing a lot3')
    #     AcceptedPass.objects.create(accpassid=99999, lotid=self.lot, passid=self.pas3, allottedstalls=2)
    #     self.assertEqual(predictor.get_capacity(99990), 176)

    def assertListAlmostEqual(self, list1, list2, places):
        for index in range(len(list1)):
            self.assertAlmostEqual(list1[index], list2[index], places)

    def test_replace_dow_avg_row(self):
        df = pd.DataFrame()

        correct_df = pd.DataFrame()
        correct_df['LotID'] = [1]
        correct_df['1'] = [0.]
        correct_df['2'] = [0.]
        correct_df['3'] = [0.]
        correct_df['4'] = [0.]
        correct_df['5'] = [0.25]
        correct_df['6'] = [0.]
        correct_df['7'] = [0.]

        settings = {'lot_id' : 1, 'dow' : 5, 'percent_full' : 0.25}
        output_df = predictor._replace_dow_avg_row(df, **settings)
        
        self.assertTrue(output_df.equals(correct_df))

        df = pd.DataFrame()
        df['LotID'] = [0,1,2]
        df['1'] = [0.,0.,0.]
        df['2'] = [0.,0.,0.]
        df['3'] = [0.,0.,0.]
        df['4'] = [0.,0.,0.]
        df['5'] = [0.,0.,0.]
        df['6'] = [0.,0.,0.]
        df['7'] = [0.,0.,0.]

        correct_df = pd.DataFrame()
        correct_df['LotID'] = [0,1,2]
        correct_df['1'] = [0.,0.,0.]
        correct_df['2'] = [0.,0.,0.]
        correct_df['3'] = [0.,0.,0.]
        correct_df['4'] = [0.,0.,0.]
        correct_df['5'] = [0,0.25,0]
        correct_df['6'] = [0.,0.,0.]
        correct_df['7'] = [0.,0.,0.]
        settings = {'lot_id' : 1, 'dow' : 5, 'percent_full' : 0.25}
        output_df = predictor._replace_dow_avg_row(df, **settings)
        self.assertTrue(output_df.equals(correct_df))

    def test_add_data(self):
        df = pd.DataFrame()
        new_df = predictor._add_data(df, lot_id=1, timestamp=23412, capacity=150, occupancy=21, dow_avg=0.6, type=UpdateType.HALFHOUR)
        correct_df = pd.DataFrame()
        correct_df['Timestamp'] = [23412]
        correct_df['Capacity'] = [150]
        correct_df['Occupancy'] = [21]
        correct_df['DOWAvg'] = [0.6]
        self.assertTrue(new_df.equals(correct_df))

        df = pd.DataFrame()
        df['Timestamp'] = [1]
        df['Capacity'] = [2]
        df['Occupancy'] = [3]
        df['DOWAvg'] = [0.4]
        new_df = predictor._add_data(df, lot_id=1, timestamp=23412, capacity=150, occupancy=21, dow_avg=0.6, type=UpdateType.HALFHOUR)
        correct_df = pd.DataFrame()
        correct_df['Timestamp'] = [1, 23412]
        correct_df['Capacity'] = [2, 150]
        correct_df['Occupancy'] = [3, 21]
        correct_df['DOWAvg'] = [0.4, 0.6]
     
        self.assertTrue(new_df.equals(correct_df))

    def test_preprocess_data(self):
        df = pd.DataFrame()
        ts = datetime.fromisoformat('2011-01-01').timestamp()
        df['Timestamp'] = [ts]
        df['DOWAvg'] = [0.6]
        df['Occupancy'] = [4]
        df['Capacity'] = [10]

        correct_df = pd.DataFrame()
        # correct_df['Timestamp'] = [1.]
        correct_df['DayOfWeek'] = [6 / 7] # saturday
        correct_df['Month'] = [1 / 12]
        correct_df['Day'] = [1/31]
        correct_df['Hour'] = [0/24]
        correct_df['DOWAvg'] = [0.6]
        correct_df['Capacity'] = [10 / 1000]
        correct_df['PercentFull'] = [0.4]

        processed_df = predictor._preprocess_data(df, type=UpdateType.HALFHOUR)
        self.assertTrue(processed_df.equals(correct_df))

    # note: this test will need to be updated in case of a new model
    def test_predict_01(self):
        data = torch.tensor([[0.4285714328, 0.8333333135, 0.6129032373, 0.6250000000, 0.2138103545,
         2.9370000362, 0.4140279293],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.6250000000, 0.2138103545,
         2.9370000362, 0.3953013420],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.6666666865, 0.2138103545,
         2.9370000362, 0.3690840900],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.6666666865, 0.2138103545,
         2.9370000362, 0.3449234068],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7083333135, 0.2138103545,
         2.9370000362, 0.3215645552],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7083333135, 0.2138103545,
         2.9370000362, 0.2964873910],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7500000000, 0.2138103545,
         2.9370000362, 0.2803853750],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7500000000, 0.2138103545,
         2.9370000362, 0.2530612350],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7916666865, 0.2138103545,
         2.9370000362, 0.2427825183],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.7916666865, 0.2138103545,
         2.9370000362, 0.2190769762],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.8333333135, 0.2138103545,
         2.9370000362, 0.2076621950],
        [0.4285714328, 0.8333333135, 0.6129032373, 0.8333333135, 0.2138103545,
         2.9370000362, 0.1937642843]], dtype=torch.float32)
        correct_predictions = [0.17875233, 0.16832092, 0.15851006, 0.14918417, 0.14063129, 0.13364917]
        test_predictions = predictor._predict(data, type=UpdateType.HALFHOUR)
        self.assertListAlmostEqual(correct_predictions, test_predictions, places=1)


    def test_predict_02(self):
        data = torch.tensor([[0.1428571492, 0.9166666865, 0.4516128898, 0.4583333433, 0.2215256244,
         1.1940000057, 0.3961474001],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5000000000, 0.2215256244,
         1.1940000057, 0.4530988336],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5000000000, 0.2215256244,
         1.1940000057, 0.4974874258],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5416666865, 0.2215256244,
         1.1940000057, 0.5067001581],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5416666865, 0.2215256244,
         1.1940000057, 0.5201004744],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5833333135, 0.2215256244,
         1.1940000057, 0.5117253065],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.5833333135, 0.2215256244,
         1.1940000057, 0.4765494168],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.6250000000, 0.2215256244,
         1.1940000057, 0.4639866054],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.6250000000, 0.2215256244,
         1.1940000057, 0.4405360222],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.6666666865, 0.2215256244,
         1.1940000057, 0.4078727067],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.6666666865, 0.2215256244,
         1.1940000057, 0.3534338474],
        [0.1428571492, 0.9166666865, 0.4516128898, 0.6666666865, 0.2215256244,
         1.1940000057, 0.3004308045]], dtype=torch.float32)
        correct_predictions = [0.25782892, 0.21633579, 0.17786025, 0.13918479, 0.10410061, 0.07045972]
        test_predictions = predictor._predict(data, type=UpdateType.HALFHOUR)
        self.assertListAlmostEqual(correct_predictions, test_predictions, places=1)

#---------------------MEDIA TESTING-----------------------#
#TODO: Test media utility functions
class MediaTests(APITestCase):
    @classmethod
    def setUpClass(self):
        self.client = APIClient()
        super(MediaTests,self).setUpClass()
        self.filedata1 = {
            'description' : 'U',
            'file' : '',
            'folder': ''
        }    
        
        # Set up an authenticated token for tests
        self.tester_creds={"username": "tester1", "password":"tester1"}
        self.tester1 = User.objects.create_superuser(username="tester1", password="tester1", is_superuser=True)
        authenticate_view = reverse('cookie_token_obtain_pair')
        response_token = self.client.post(authenticate_view, data=self.tester_creds)
        self.token = response_token.json()['access'] 

    def basicSystemSetupTest(self):
        # Creating (POST) a system data
        view = reverse('systeminfo')
        response = self.client.post(view, data=self.filedata1, format='json', **{"HTTP_AUTHORIZATION" :f"Bearer {self.token}"})
        self.assertEqual(response.status_code, 200)
        self.assertTrue(filecmp.cmp(f".{settings.MEDIA_URL}systeminfo/system_info.json", f".{settings.MEDIA_URL}systeminfo/system_info.json", shallow=False))


    
