from django.urls import path, re_path
from . import views
from django.conf import settings
from django.conf.urls.static import static
from rest_framework import permissions
from drf_yasg.views import get_schema_view
from drf_yasg import openapi

schema_view = get_schema_view(
   openapi.Info(
      title="UPark API",
      default_version='v1',
      description="Parking Lot Metrics for the University of Utah",
   ),
   public=True,
   permission_classes=[permissions.AllowAny],
)

urlpatterns = [

   re_path(r'^swagger(?P<format>\.json|\.yaml)$', schema_view.without_ui(cache_timeout=0), name='schema-json'),
   re_path(r'^swagger/$', schema_view.with_ui('swagger', cache_timeout=0), name='schema-swagger-ui'),
   re_path(r'^redoc/$', schema_view.with_ui('redoc', cache_timeout=0), name='schema-redoc'),

   # Parking Lot CRUD
   path('lot/', views.ListParkingLotsView.as_view(), name='lot-list'), #list all parking lots, add parking lot to system
   re_path(r'^lot/(?P<pk>\d+)/$', views.DetailParkingLotView.as_view(), name='lot-pk'), #read/update single parking lot
   path('lotcount/<int:pk>/', views.CountParkingLotView.as_view(), name='lotcount-pk'), #update single parking lot count, get used stalls value
   # TODO: maybe we don't want to have the device rely on this endpoint when submitting changes?

   path('lot/<int:pk>/sensors/', views.ListLotSensorsView.as_view(), name='lot-sensor-list'), #list all sensors on a single lot
   path('lot/<int:pk>/acceptedpasses/', views.ListLotAcceptedPassesView.as_view(), name='lot-accpass-list'), #get all accpasses from a single lot
   path('lot/<int:pk>/availableacceptedpasses/', views.ListLotAvailableAcceptedPassesView.as_view(), name='lot-available-accpasstoadd-list'), #get all accpasses that can still be added to a specific lot
   path('lot/acceptedpasses/', views.ListAllLotAcceptedPassesView.as_view(), name='lot-all-accpass-list'), # get a map from lot numbers to accepted passes
   path('lot/<int:pk>/entranceexits/', views.ListLotEnExesView.as_view(), name='lot-enex-list'), #list all entranceExits from a single lot

   path('lot/<int:pk>/percentfull/', views.ParkingLotPercentFullView.as_view(), name='lot-percent-full'), #a percentage of the fullness of a lot given a lotid
   path('lot/percentagesfull/', views.ListParkingLotPercentFullView.as_view(), name='list-lots-percent-full'), #the percentages of the fullness of all lots

   # Parking Lot Misc.
   path('lot/<int:pk>/capacity/', views.ParkingLotCapacityView.as_view(), name='lot-capacity'), #stall capacity of a given lot (total across all passes)
   path('lot/maxcapacity/',views.LotMaxCapacityView.as_view(), name='lot-maxcapacity'), #Returns the highest capacity of all the lots
   path('lot/<int:pk>/stallsremaining/', views.ParkingLotStallsRemainingView.as_view(), name='lot-stalls-remaining'), #total value of stalls remaining in a given lot
   path('lotcomplete/', views.MegaListParkingLotsView.as_view(), name='lot-list-mega'), #list all parking lots, add parking lot to system
   path('lotcomplete/<int:pk>/', views.MegaSingleParkingLotView.as_view(), name='lot-mega'), #list all parking lots, add parking lot to system

   # Lot prediction information
   path('lot/<int:pk>/prediction/halfhours/', views.PredictionHalfHourView.as_view()), #get next x half-hour's data from prediction model
   path('lot/<int:pk>/prediction/week/', views.PredictionWeekView.as_view()), # get next x day's data from prediction model

   # Dummy
   path('dummy/', views.DummyView.as_view()), # a dummy view that doesn't do anything.

   # Sensor CRUD
   path('sensor/create/', views.ListSensorsView.as_view(), name='sensor-create'), #create new sensor
   path('sensor/', views.ListSensorsView.as_view(), name='sensor-list'), #list all sensors
   path('sensor/<pk>/', views.DetailSensorView.as_view(), name='single-sensor'), #show or update single sensor

   # Parking Pass CRUD
   path('pass/create/', views.ListPassesView.as_view(), name='pass-create'), #create new pass
   path('pass/', views.ListPassesView.as_view(), name='pass-list'), #list all passes
   path('pass/<int:pk>/', views.DetailPassView.as_view(), name='single-pass'), #show or update single pass

   # Accepted Pass CRUD
   path('acceptedpass/create/', views.ListAcceptedPassesView.as_view(), name='accepted-pass-create'), #create a new accepted pass type for a parking lot
   path('acceptedpass/', views.ListAcceptedPassesView.as_view(), name='accepted-pass-list'), #list all accepted passes
   path('acceptedpass/<int:pk>', views.DetailAcceptedPassView.as_view(), name='single-accepted-pass'), #show or update single accepted pass

   # Entrance Exit CRUD
   path('enex/create/', views.ListEnExView.as_view(), name='enex-create'), #create a new entrance/exit for a parking lot
   path('enex/', views.ListEnExView.as_view(), name='enex-list'), #list all entrance/exits
   path('enex/<int:pk>', views.DetailEnExView.as_view(), name='single-enex'), #show or update a single extrance/exit

   # TODO:  #get all accpasses from a given pass # maybe this could be get all lots given a passID instead, might be more useful

   # Post setup information
   path('setup/mediastorage/', views.MediaStorageView.as_view(), name='media-storage'),
   path('setup/systeminfo/', views.InitSystemInfoView.as_view(), name='system-info'),

   # Admin announcements
   path('adminannouncement/<int:pk>/', views.AdminAnnouncementView.as_view(), name='admin-announcement'), #get or set an announcement for a specific parking lot, <pk> of 0 corresponds to the whole system

   # Commuter User Lot and Building Search
   path('lotsearch/<filters>/', views.LotSearchView.as_view(), name='lot-search'),
   path('lotsearch/<filters>/<terms>/', views.LotSearchView.as_view(), name='lot-search-terms'), #Search for a parking lot in the database based on a query
   path('lotsearch/<lat>/<long>/<filters>/', views.LotSearchByTermsAndLocationView.as_view(), name='lot-location-search'),
   path('lotsearch/<lat>/<long>/<filters>/<terms>/', views.LotSearchByTermsAndLocationView.as_view(), name='lot-location-search-terms'), #Search for closest parking lots in the database based on a search query and coordinates

   path('buildingsearch/<filters>/<terms>/', views.BuildingSearchView.as_view(), name='building-search-terms'), #Search for a building in the lot_data_spreadsheets/building.csv file based on a query
   path('buildingsearch/<filters>/', views.BuildingSearchView.as_view(), name='building-search'),

   path('lotsandbuildings/', views.ListParkingLotsAndBuildingsView.as_view(), name='lots-and-buildings-list'),

   # Statistics
   path('statistics/percent/<lots>/<date>', views.StatisticsPercentDayView.as_view(), name='stats-percent-day'),
   path('statistics/percent/<metric>/<lots>/<start_date>/<end_date>', views.StatisticsPercentMultiDayView.as_view(), name='stats-percent-multi-day'),

   #Cookied HttpOnly Tokens
   path('token/', views.CookieTokenObtainPairView.as_view(), name='cookie_token_obtain_pair'),
   path('token/refresh/', views.CookieTokenRefreshView.as_view(), name='cookie_token_refresh'),
   path('token/blacklistme/', views.CookieBadTokenView.as_view(), name='cookie_bad_token'),
]

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
