from operator import contains
from django.conf import settings
from django.db.models import F, Q
from django.http import HttpResponse, JsonResponse
from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import BasePermission, SAFE_METHODS, IsAdminUser, IsAuthenticated
from rest_framework.parsers import MultiPartParser, FormParser
from django.shortcuts import get_object_or_404
from drf_yasg.utils import swagger_auto_schema
from django.db.models import Sum
from django.core import serializers
import pandas as pd
import pytz #for timezone info

from library.predictor import UpdateType

from . import _upark_utilities
import pytz #for timezone info

from datetime import datetime, timedelta
import json
import csv
import os

from api.models import AcceptedPass, EntranceExit, Pass, User, ParkingLot, CvImageSensor
from api.serializers import *

import library.predictor as predictor


#---------------------PERMISSION CLASSES-----------------------#

class AllowReadOnlyMethods(BasePermission):
    """
    Rest framework permission type to
    allow only if the request's methods is a READONLY (get, options, head)
    """

    def has_permission(self, request, view):
        return request.method in SAFE_METHODS

class IsSensorUser(BasePermission):
    """
    Rest framework permission type to
    allow a request only if the request's user is a sensor
    """
    message = 'This action is permitted to SensorUser'

    def has_permission(self, request, view):
        return request.user.is_sensor

#---------------------PARKING LOT VIEW CLASSES-----------------------#

class CountParkingLotView(APIView):
    """
    Request Body example: {usedstalls:-10}
    An API "view"/route that sees usedstalls (R) of a parkinglot and adjusts only the usedstalls of a parking lot (U)
    """
    serializer_class = CountParkingLotSerializer
    permission_classes = [IsAdminUser | IsSensorUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False), 400: "Couldn't get used stalls"})
    def get(self, request, pk):
        """Read a lot's usedstalls value"""
        item = get_object_or_404(ParkingLot, pk=pk)
        serializer = self.serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "OK", 400: "Bad request, couldn't update used stalls"})
    def patch(self, request, pk):
        """Patch/update a lot's usedstalls by an amount"""
        amount = request.data["usedstalls"]
        item = get_object_or_404(ParkingLot, pk=pk)
        data = {
            "usedstalls": item.usedstalls + int(amount),
            "timecountupdated": datetime.now()
        }
        serializer = self.serializer_class(item, data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

class DetailParkingLotView(APIView):
    """An API route that provides all information about a specific parking lot (R) and allows for editing of parking lot information (U, D)."""
    serializer_class = GetAllParkingLotSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False), 400: "Couldn't get parking lot"})
    def get(self, request, pk):
        """Get information on a specific parking lot (R)"""
        item = get_object_or_404(ParkingLot, pk=pk)
        serializer = self.serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Updated Parking lot successfully", 400: "Couldn't update parking lot"})
    def patch(self, request, pk):
        """
        Adjust values of a specific parking lot (U)
            - example request body: {"name": "lotname", "latitude": float, "longitude": float, "totalstalls": int, "usedstalls": int}
        """
        item = get_object_or_404(ParkingLot, pk=pk)
        data = {
            "lotid": item.pk,
            "name": request.data["name"],
            "latitude": request.data["latitude"],
            "longitude": request.data["longitude"],
            "usedstalls": request.data["usedstalls"],
            "timecountupdated": datetime.now(pytz.utc),
        }
        serializer = self.serializer_class(item, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot delete successful", 400: "Parking lot delete failed"})
    def delete(self, request, pk):
        """Delete a single parking lot"""
        # EntranceExit.objects.filter(lotid=pk).delete()
        ParkingLot.objects.filter(pk=pk).delete()
        return Response(status=status.HTTP_200_OK)

class ListParkingLotsView(APIView):
    """API for a list of all parking lots and their properties"""
    serializer_class = GetAllParkingLotSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=True), 400: "Couldn't get parking lots"})
    def get(self, request):
        items = ParkingLot.objects.all()
        serializer = self.serializer_class(items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot creation successful", 400: "Parking lot creation failed"})
    def post(self, request):
        """
        Add a new parking lot to the system
         - Restricted to Admin/Staff only
         - All fields must be met from the request body
        """
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

################ MEGA LOT VIEWS (url:api/lotcomplete) ##########################
class MegaListParkingLotsView(APIView):
    serializer_class = GetAllParkingLotSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]
    def get(self, request):

        data = []
        lots = ParkingLot.objects.all().values()



        for lot in lots.iterator():
            accpasses = list(AcceptedPass.objects.all().filter(lotid=lot['lotid'])
                        .select_related("passid__shortname", "passid__name")
                        .values('accpassid', 'allottedstalls', shortname=F('passid__shortname'), name=F('passid__name')))
            enexcount = EntranceExit.objects.filter(lotid=lot['lotid']).count()

            capacity = 0
            for accpass in accpasses:
                capacity += accpass['allottedstalls']
            try:
                percentfull = lot['usedstalls'] / capacity
            except ZeroDivisionError:
                percentfull = "unavailable"
            #  Get remaining stalls
            total_stalls = sum([int(accpass['allottedstalls']) for accpass in accpasses])
            remaining_stalls=total_stalls - lot["usedstalls"]
            obj = {"lot": lot, "accpasses": accpasses, "enexcount": enexcount, "percentfull": percentfull, "stallsremaining":remaining_stalls}
            data.append(obj)
        return Response(data,  status=status.HTTP_200_OK)

class MegaSingleParkingLotView(APIView):

    serializer_class = GetAllParkingLotSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]
    @swagger_auto_schema(responses={200: serializer_class(many=False), 400: "Couldn't get parking lot"})
    def get(self, request, pk):
        """Get complete data related to a given Parking Lot

        Args:
            request (HTTPRequest):  The request's data
            pk (_Number_): primary key of the parking lot to look for (aka lotid)

        Returns:
            Response: 404 if not found, o.w. the whole data of the parking lot
        """

        lot = get_object_or_404(ParkingLot, pk=pk)

        accpasses = list(AcceptedPass.objects.all().filter(lotid=lot.lotid)
                    .select_related("passid__shortname", "passid__name")
                    .values('accpassid', 'allottedstalls', shortname=F('passid__shortname'), name=F('passid__name')))
        enexes = EntranceExit.objects.filter(lotid=lot.lotid)
        enexcount = len(enexes)
        capacity = 0

        for accpass in accpasses:
            capacity += accpass['allottedstalls']
        try:
            percentfull = lot.usedstalls / capacity
        except ZeroDivisionError:
            percentfull = "unavailable"
        enex_metadata = GetAllEnExSerializer(enexes, many=True).data
        # Add associated sensor data to enex
        for enex in enex_metadata:
            sensor = CvImageSensor.objects.filter(enid=enex['enid'])
            if len(sensor) == 0:
                continue
            enex['sensor'] = GetAllCvSensorSerializer(sensor[0],many=False).data #Each enex has exactly one sensor (or none)
        lot_serializer=self.serializer_class(lot) #serialize the parking lot model object to a json printable
        # Get remaining stalls
        total_stalls = sum([int(accpass['allottedstalls']) for accpass in accpasses])
        remaining_stalls=total_stalls - lot.usedstalls
        obj = {"lot": lot_serializer.data, "accpasses": accpasses, "enexes":enex_metadata, "enexcount": enexcount, "percentfull": percentfull, "stallsremaining":remaining_stalls}
        return Response(obj,  status=status.HTTP_200_OK)

###############################################################################

class ListLotSensorsView(APIView):
    """Endpoint for a list of all sensors given a single lotID"""
    serializer_class = GetAllCvSensorSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=True)})
    def get(self, request, pk):
        """
        List all sensors for a given parking lot with a specific lotID (R).
        """
        lotEnExIds = EntranceExit.objects.all().filter(lotid=pk).values_list('pk', flat=True)
        sensors = CvImageSensor.objects.all().filter(enid__in=lotEnExIds)
        serializer = self.serializer_class(sensors, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

class ParkingLotPercentFullView(APIView):
    """API for a single lot's percentage full"""
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    def get(self, request, pk):
        """
        Get the percentage full of a parking lot by specific lotid
        """
        capacity = 0
        for obj in AcceptedPass.objects.filter(lotid=pk):
            capacity += obj.allottedstalls
        if (capacity <= 0):
            return Response({"detail":"Capacity is invalid for this lot."}, status = status.HTTP_400_BAD_REQUEST)
        usedStallsDict = ParkingLot.objects.filter(lotid=pk).values('usedstalls')[0]
        percentageFull = usedStallsDict['usedstalls'] / capacity
        if (percentageFull < 0 or percentageFull > 1):
            return Response({"detail":"Invalid percentage full (< 0 or > 1)."}, status = status.HTTP_400_BAD_REQUEST)
        return Response(percentageFull)

class ListParkingLotPercentFullView(APIView):
    """Endpoint for a list of percent full data for every parkinglot"""
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    def get(self, request):
        """
        Get the percentage full for every lot in a list
        """
        stallTotals = (AcceptedPass.objects.values('lotid').annotate(totalstalls=Sum('allottedstalls')))
        usedStalls = (ParkingLot.objects.values('lotid', 'usedstalls'))

        lotPercentages = []
        # schema: list of {lotid: 1, percentage: 90% }
        for t in stallTotals:
            curLot = {}
            curLot['percentage'] = usedStalls.filter(lotid=t['lotid'])[0]['usedstalls'] / t['totalstalls']

            # Check if the percent full calculation is valud
            if (curLot['percentage'] > 1 or curLot['percentage'] < 0):
                return Response({"detail":"Invalid percentage full (< 0 or > 1) at Lot " + str(usedStalls.filter(lotid=t['lotid'])[0]['lotid'])+ "."}, status = status.HTTP_400_BAD_REQUEST)

            lotPercentages.append(curLot)

        return Response(lotPercentages)

class ListLotAcceptedPassesView(APIView):
    """Endpoint for a list of all AcceptedPasses given a single lotID"""
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema()
    def get(self, request, pk):
        """
        List all AcceptedPasses for a given parking lot with a specific lotID (R).
        """
        accPasses = (AcceptedPass.objects.all().filter(lotid=pk)
                        .select_related("passid__shortname", "passid__name")
                        .values('accpassid', 'allottedstalls', shortname=F('passid__shortname'), name=F('passid__name')))
        data = list(accPasses)
        return Response(data)

class ListLotAvailableAcceptedPassesView(APIView):
    """Endpoint for a list of all AcceptedPasses that are available to add given a single lotID"""
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema()
    def get(self, request, pk):
        """
        List all AcceptedPasses that can be added for a given parking lot with a specific lotID (R).        """

        accPasses = (AcceptedPass.objects.all().filter(lotid=pk)
                        .select_related("passid__shortname", "passid__name")
                        .values('accpassid', 'allottedstalls', shortname=F('passid__shortname'), name=F('passid__name')))
        availablePasses = (Pass.objects.all().exclude(passid__in=accPasses.values_list('passid'))).values('passid', 'shortname', 'name')
        data = list(availablePasses)
        return Response(data)


class ListAllLotAcceptedPassesView(APIView):
    """ Endpoint to get a mapping from lots to their accepted passes"""
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]
    swagger_auto_schema()
    def get(self, request):
        accPasses = list(AcceptedPass.objects.all()
                                .select_related("passid_shortname")
                                .values("lotid", shortname=F('passid__shortname')))

        result = {}

        for accPass in accPasses:
            if accPass["lotid"] not in result:
                result[accPass["lotid"]] = []
            result[accPass["lotid"]].append(accPass["shortname"])

        return Response(result)



class ListLotEnExesView(APIView):
    """Endpoint for a list of all EntranceExits given a single lotID"""
    # serializer_class = GetAllEnExSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema()
    def get(self, request, pk):
        """
        List all EntranceExits for a given parking lot with a specific lotID (R).
        """
        enexes = (EntranceExit.objects.all()
                    .select_related("enid").filter(lotid=pk)
                    .values("enid", 'latitude', 'longitude','lotid'))
        data = list(enexes)
        return Response(data)

#---------------------ENTRANCE/EXIT VIEW CLASSES-----------------------#

class DetailEnExView(APIView):
    """An API route that provides all information about a specific Entrance/Exit (R) and allows for editing of an Entrance/Exit (U, D)."""
    serializer_class = EnExSerializer
    get_serializer_class = GetAllEnExSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False)})
    def get(self, request, pk):
        """Get information on a specific EntranceExit (R)"""
        item = get_object_or_404(EntranceExit, pk=pk)
        serializer = self.get_serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Entrance/Exit update successful", 400: "Entrance/Exit update failed"})
    def patch(self, request, pk):
        """
        Adjust values of a specific Entrance/Exit (U)
         - example request body: {"lotid": int, "longitude": float, "latitude": float}
        """
        item = get_object_or_404(EntranceExit, pk=pk)
        data = {
            "lotid": request.data["lotid"],
            "longitude": request.data["longitude"],
            "latitude": request.data["latitude"],
        }
        serializer = self.serializer_class(item, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot delete successful", 400: "Parking lot delete failed"})
    def delete(self, request, pk):
        """Delete a single Entrance/Exit"""
        EntranceExit.objects.filter(pk=pk).filter().delete()
        return Response(status=status.HTTP_200_OK)

class ListEnExView(APIView):
    """API for a list of all entrance/exits and their properties """
    serializer_class = EnExSerializer
    get_serializer_class = GetAllEnExSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=True)})
    def get(self, request):
        """
        List all entrance/exits for the entire parking system (R).
        """
        items = EntranceExit.objects.all()
        serializer = self.get_serializer_class(items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Entrance/Exit creation successful", 400: "Entrance/exit creation failed"})
    def post(self, request):
        """
        Add a new entrance/exit to the system (C)
         - Restricted to Admin/Staff only
         - All fields must be met from the request body
        """
        data = dict(request.data)
        data["lotid"] = str(data['lotid'])

        serializer = self.serializer_class(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

#---------------------PARKING PASS VIEW CLASSES-----------------------#

class DetailPassView(APIView):
    """An API route that provides all information about a specific parking pass (R) and allows for editing of parking pass (U, D)."""
    serializer_class = ParkingPassSerializer
    get_serializer_class = GetAllParkingPassSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False)})
    def get(self, request, pk):
        """Get information on a specific parking pass (R)"""
        item = get_object_or_404(Pass, pk=pk)
        serializer = self.get_serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class,responses={200: "Pass update successful", 400: "Pass update failed"})
    def patch(self, request, pk):
        """
        Adjust values of a specific pass (U)
         - example request body: {"shortname": "U", "name": "University"}
        """
        item = get_object_or_404(Pass, pk=pk)
        data = {
            "passid": item.pk,
            "shortname": request.data["shortname"],
            "name": request.data["name"],
        }
        serializer = self.serializer_class(item, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot delete successful", 400: "Parking lot delete failed"})
    def delete(self, request, pk):
        """Delete a single pass"""
        # AcceptedPass.objects.filter(passid=pk).delete()
        Pass.objects.filter(pk=pk).delete()
        return Response(status=status.HTTP_200_OK)

class ListPassesView(APIView):
    """API for a list of all passes and their properties"""
    serializer_class = ParkingPassSerializer
    get_serializer = GetAllParkingPassSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=True)})
    def get(self, request):
        """List all passes (R)"""
        items = Pass.objects.all()
        serializer = self.get_serializer(items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class,responses={200: "Pass creation successful", 400: "Pass creation failed"})
    def post(self, request):
        """
        Add a new pass to the system
         - Restricted to Admin/Staff only
         - All fields must be met from the request body
        """
        data = {
            "shortname": request.data["shortname"],
            "name": request.data["name"]
        }
        serializer = self.serializer_class(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

#---------------------ACCEPTED PASS VIEW CLASSES-----------------------#

class DetailAcceptedPassView(APIView):
    """An API route that provides all information about a specific accepted pass (R) and allows for editing of accepted pass (U, D)."""
    serializer_class = AcceptedPassSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False)})
    def get(self, request, pk):
        """Get information on a specific accepted pass (R)"""
        item = get_object_or_404(AcceptedPass, pk=pk)
        serializer = self.serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class,responses={200: "Accepted pass update successful", 400: "Accepted pass update failed"})
    def patch(self, request, pk):
        """
        Adjust values of a specific accepted pass (U)
         - example request body: {"lotid": int, "passid": int, "allottedstalls": int}
        """
        item = get_object_or_404(AcceptedPass, pk=pk)
        data = {
            "lotid": request.data["lotid"],
            "passid": request.data["passid"],
            "allottedstalls": request.data["allottedstalls"],
        }
        serializer = self.serializer_class(item, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot delete successful", 400: "Parking lot delete failed"})
    def delete(self, request, pk):
        """Delete a single accepted pass"""
        AcceptedPass.objects.filter(pk=pk).filter().delete()
        return Response(status=status.HTTP_200_OK)

class ListAcceptedPassesView(APIView):
    """API for a list of all passes and their properties """
    serializer_class = AcceptedPassSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=True)})
    def get(self, request):
        """
        List all accepted passes for the entire parking system (R).
        """
        items = AcceptedPass.objects.all()
        serializer = self.serializer_class(items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Accepted pass creation successful", 400: "Accepted pass creation failed"})
    def post(self, request):
        """
        Add a new accepted pass to the system (C)
         - Restricted to Admin/Staff only
         - All fields must be met from the request body
        """
        data = dict(request.data)
        data["lotid"] = str(data['lotid'])
        data["passid"] = str(data['passid'])

        serializer = self.serializer_class(data=data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

#---------------------SENSOR VIEW CLASSES-----------------------#

class DetailSensorView(APIView):
    """An API route that provides all information about a specific Sensor."""
    serializer_class = CvSensorSerializer
    get_serializer_class = GetAllCvSensorSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: serializer_class(many=False)})
    def get(self, request, pk):
        """Get information about a specific sensor (R)"""
        item = get_object_or_404(CvImageSensor, pk=pk)
        serializer = self.get_serializer_class(item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Sensor update successful", 400: "Sensor update failed"})
    def patch(self, request, pk):
        """
        Adjust values of a specific sensor (U), partial accepted
         - Restricted to Admin/Staff only
         - example request body: {"enid: int, isentranceside: int"}
        """
        item = get_object_or_404(CvImageSensor, pk=pk)
        data = {
            "user": item.user.pk,
            "sensorid": item.sensorid,
            "enid": request.data["enid"],
            "timelastupdate": item.timelastupdate,
            "isentranceside": request.data["isentranceside"],
            "imageurl": item.imageurl,
            "imagetime": item.imagetime
        }
        serializer = self.serializer_class(item, data=data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Parking lot delete successful", 400: "Parking lot delete failed"})
    def delete(self, request, pk):
        """Delete a single sensor (D)"""
        CvImageSensor.objects.filter(pk=pk).delete()
        return Response(status=status.HTTP_200_OK)

class ListSensorsView(APIView):
    """API for creating a new sensor (C) and listing all sensors that exist (R)"""
    serializer_class = CvSensorSerializer
    get_serializer_class = GetAllCvSensorSerializer
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(responses={200: get_serializer_class(many=True)})
    def get(self, request):
        """List all sensors (R)"""
        items = CvImageSensor.objects.all()
        serializer = self.get_serializer_class(items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @swagger_auto_schema(request_body=serializer_class, responses={200: "Sensor creation successful", 400: "Sensor creation failed"})
    def post(self, request):
        """Create new sensor (C)"""
        # get max userID
        max_userId = User.objects.all().values_list('id').order_by('-id').first()[0]
        # create new user object
        new_sensor_user = User.objects.create_user(
                                username=f'sensor{max_userId+1}',
                                password=f'sensor{max_userId+1}', #TODO: make this more secure
                                is_sensor=True)
        # create new sensor object
        new_sensor_data = {
            "user": new_sensor_user.pk, #user has been created above
            "enid": request.data["enid"],
            "timelastupdate": None,
            "isentranceside": request.data["isentranceside"],
            "imageurl": None,
            "imagetime": None
        }

        serializer = self.serializer_class(data=new_sensor_data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

#---------------------API UPARK SETUP CLASSES-----------------------#

# API endpoint for uploading media
class MediaStorageView(APIView):
    """
           Upload media files to the system
    """
    serializer_class = MediaStorageSerializer
    permission_classes = [IsAdminUser]
    parser_classes = [MultiPartParser, FormParser]

    # ***Swagger documentation is currently hidden***
    @swagger_auto_schema(auto_schema=None)
    def get(self, request):
        return Response("GET API")

    @swagger_auto_schema(auto_schema=None, responses={200: "File uploaded", 400: "File upload failed"})
    def post(self, request, format=None):
        # name = request.POST.get("systemName")
        # data = {"systemName": name}

        # json_string = json.dumps(data)
        # # Directly from dictionary
        # with open('./media/systeminfo/system_info.json', 'w') as outfile:
        #     json.dump(json_string, outfile)

        # # Using a JSON string
        # with open('./media/systeminfo/system_info.json', 'w') as outfile:
        #     outfile.write(json_string)
        print(request.data)
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

# API for uploading miscellaneous info to a textfile:
class InitSystemInfoView(APIView):
    """API Endpoint for uploading miscellaneous setup info into a json file.
    System name, system short name, default timezone, etc..
    """
    permission_classes = [IsAdminUser]

    # ***Swagger documentation is currently hidden***
    @swagger_auto_schema(auto_schema=None)
    def get(self, request):
        try:
            with open(f".{settings.MEDIA_URL}systeminfo/system_info.json", 'r') as infile:
                sysInfo = json.load(infile)
                return JsonResponse(sysInfo, status=status.HTTP_200_OK)
        except IOError as e:
            return Response("Read system file error",status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(auto_schema=None, responses={200: "System info uploaded", 400: "System info upload failed"})
    def post(self, request):
        json_string = json.dumps(request.data)
        try:
            with open(f".{settings.MEDIA_URL}systeminfo/system_info.json", 'w') as outfile:
                outfile.write(json_string)
            return Response( status=status.HTTP_200_OK)
        except IOError as e:
            print(f'error occured: {e}')
            return Response("Init system file error",status=status.HTTP_400_BAD_REQUEST)

#---------------------Admin Annoucements API Endpoints-----------------------#

# API for uploading admin annoucements for parkings lots and the system:
class AdminAnnouncementView(APIView):
    """API Endpoint for uploading admin announcement info into a json file.
       Lot number, announcement, expiration date, etc.
    """
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    # ***Swagger documentation is currently hidden***
    @swagger_auto_schema(auto_schema=None)
    def get(self, request, pk):
        try:
            with open(f".{settings.MEDIA_URL}systeminfo/announcements.json", 'r+') as outfile:
                try:
                    data = json.load(outfile)
                except:
                    outfile.write(json.dumps({"announcements": [{"lotid": pk, "announcement": "N/A","startDate": "N/A","endDate": "N/A"}]}))
                    return Response({'lotid':pk,'announcement': 'N/A',"startDate:": "N/A","endDate": "N/A"})

                for parkinglot in data["announcements"]:
                    if int(parkinglot['lotid']) == pk:
                        return Response({"lotid":parkinglot['lotid'],"announcement":parkinglot['announcement'],"startDate":parkinglot['startDate'],"endDate":parkinglot['endDate']})

                return Response({'lotid':pk,'announcement': 'N/A','startDate': 'N/A',"endDate":"N/A"})
        except IOError as e:
            return Response("Announcement create error",status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(auto_schema=None, responses={200: "Annoucement uploaded", 400: "Announcement upload failed"})
    def post(self, request, pk):
        json_string = request.data
        try:
            if not os.path.exists(f".{settings.MEDIA_URL}systeminfo/announcements.json"):
                with open(f".{settings.MEDIA_URL}systeminfo/announcements.json", 'w+'): pass
            with open(f".{settings.MEDIA_URL}systeminfo/announcements.json", 'r+') as outfile:
                try:
                    data = json.load(outfile) # Try to load existing json file
                    for parkinglot in data["announcements"]:
                        if int(parkinglot['lotid']) == pk:
                            parkinglot['announcement'] =  request.data['announcement']
                            parkinglot['startDate'] =  request.data['startDate']
                            parkinglot['endDate'] =  request.data['endDate']
                            outfile.seek(0)
                            json.dump(data, outfile)
                            return Response(status=status.HTTP_200_OK)
                    data["announcements"].append({"lotid": pk, "announcement": request.data['announcement'],"startDate": request.data['startDate'],"endDate": request.data['endDate']})
                    outfile.seek(0)
                    json.dump(data, outfile)
                    return Response(status=status.HTTP_200_OK)
                except:
                    json_string = {"announcements": [{"lotid": pk, "announcement": request.data['announcement'],"startDate": request.data['startDate'],"endDate": request.data['endDate']}]}
                    outfile.write(json.dumps(json_string))
                    return Response(status=status.HTTP_200_OK)
        except IOError as e:
            return Response("Annoucement write error",status=status.HTTP_400_BAD_REQUEST)

    @swagger_auto_schema(auto_schema=None, responses={200: "Announcement delete successful", 400: "Announcement delete failed"})
    def delete(self, request, pk):
        """Delete an announcement (D)"""
        with open(f".{settings.MEDIA_URL}systeminfo/announcements.json", 'r+') as outfile:
            try:
                if not os.path.exists(f".{settings.MEDIA_URL}systeminfo/announcements.json"):
                    with open(f".{settings.MEDIA_URL}systeminfo/announcements.json", 'w+'): pass
                data = json.load(outfile)
            except:
                #outfile.write(json.dumps({"announcements": [{"lotid": pk, "announcement": "N/A","startDate": "N/A","endDate": "N/A"}]}))
                return Response(status=status.HTTP_200_OK)

            for parkinglot in data["announcements"]:
                if int(parkinglot['lotid']) == pk:
                    parkinglot['announcement'] = "N/A"
                    parkinglot['startDate'] = "N/A"
                    parkinglot['endDate'] = "N/A"
                    outfile.seek(0)
                    outfile.truncate()
                    json.dump(data, outfile)
                    return Response(status=status.HTTP_200_OK)
            return Response(status=status.HTTP_200_OK)


#---------------------Prediction API Endpoints-----------------------#

class PredictionHalfHourView(APIView):

    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(operation_description="GET lot/{lotid}/prediction/halfhours/")
    def get(self, request, pk):
        """Get half-hour predictions for a specific lot"""

        #Call array returning function
        prediction = predictor.get_latest_prediction(int(pk), type=predictor.UpdateType.HALFHOUR)
        return JsonResponse( prediction, status=status.HTTP_200_OK, safe=False)

class PredictionWeekView(APIView):
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]
    pass

#---------------------Parking Lot Misc. Endpoints-----------------------#

class ParkingLotCapacityView(APIView):

    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(operation_description="GET lot/{lotid}/capacity/")
    def get(self, request, pk):
        return Response(predictor.get_capacity(pk), status=status.HTTP_200_OK)

class LotMaxCapacityView(APIView):
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(operation_description="GET lot/maxlotcapacity/")
    def get(self, request):
        """Returns the max capacity of the lot with highest capacity"""
        lots = ParkingLot.objects.all().values()
        max_capacity = 0
        for lot in lots.iterator():
            accpasses = AcceptedPass.objects.all().filter(lotid=lot['lotid']).values()
            total_stalls = sum([int(accpass['allottedstalls']) for accpass in accpasses])
            if max_capacity < total_stalls:
                max_capacity = total_stalls
        return Response(max_capacity, status=status.HTTP_200_OK)


class ParkingLotStallsRemainingView(APIView):

    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    @swagger_auto_schema(operation_description="GET lot/{lotid}/stallsremaining/")
    def get(self, request, pk):
        total_stalls = predictor.get_capacity(pk)
        item = get_object_or_404(ParkingLot, pk=pk)
        used_stalls = item.usedstalls
        return Response(total_stalls - used_stalls, status=status.HTTP_200_OK)

#---------------------Dummy Endpoint-----------------------#

class DummyView(APIView):
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]
    def get(self, request):
        return Response(status=status.HTTP_200_OK)

#---------------------Statistics Endpoints-----------------------#
class StatisticsPercentDayView(APIView):

    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    def get(self, request, lots, date):
        dt = datetime.fromisoformat(date)

        formatted_data = []
        lots = lots.split()
        for lot_num in lots:
            filename = predictor._get_prediction_data_filename(lot_num, UpdateType.HALFHOUR)
            if not os.path.exists(filename):
                continue
            with open(filename) as fd:
                data = pd.read_csv(fd)
                for _, row in data.iterrows():
                    row_dt = datetime.fromtimestamp(row['Timestamp'])
                    if dt.day == row_dt.day and dt.month == row_dt.month and dt.year == row_dt.year:
                        entry = {}
                        entry['time'] = int(row['Timestamp'])
                        entry[f'lot{lot_num}'] = row['Occupancy'] / row['Capacity']
                        formatted_data.append(entry)


        return Response(formatted_data, status=status.HTTP_200_OK)


class StatisticsPercentMultiDayView(APIView):
    #'statistics/percent/<metric>/<lots>/<start_date>/<end_date>',
    permission_classes = [IsAdminUser | AllowReadOnlyMethods]

    def get(self, requests, metric, lots, start_date, end_date):

        if metric != 'avg' and metric != 'max' and metric != 'min':
            return Response(status.HTTP_400_BAD_REQUEST)

        start_dt = datetime.fromisoformat(start_date)
        end_dt = datetime.fromisoformat(end_date) + timedelta(days=1) # want to include the whole end day

        # ts: {} for ts in range(int(datetime.fromisoformat('2022-03-19').timestamp()), int(datetime.fromisoformat('2022-03-22').timestamp()), 86400)
        ts_to_counts = {}
        lots = lots.split()
        for lot_num in lots:
            filename = predictor._get_prediction_data_filename(lot_num, UpdateType.HALFHOUR)
            if not os.path.exists(filename):
                continue
            with open(filename) as fd:
                data = pd.read_csv(fd)
                for _,row in data.iterrows():
                    row_dt = datetime.fromtimestamp(row['Timestamp'])
                    if start_dt <= row_dt <= end_dt:
                        ts = datetime(row_dt.year, row_dt.month, row_dt.day).timestamp()
                        # ts = int((row["Timestamp"] // 86400) * 86400) # round to day
                        if ts not in ts_to_counts:
                            ts_to_counts[ts] = {}
                        ts_to_counts[ts][f'{lot_num}count'] = ts_to_counts[ts].get(f'{lot_num}count',0) + 1
                        ts_to_counts[ts][f'{lot_num}total'] = ts_to_counts[ts].get(f'{lot_num}total',0) + row['Occupancy'] / row['Capacity']

        formatted_data = []
        for ts, dic in ts_to_counts.items():
            entry = {'day':ts}
            for lot_num in lots:
                entry[f'lot{lot_num}'] = dic.get(f'{lot_num}total', 0) / dic.get(f'{lot_num}count', 0)
            formatted_data.append(entry)

        return Response(formatted_data, status=status.HTTP_200_OK)


#---------------------API UPARK SEARCH CLASSES-----------------------#


class LotSearchView(APIView):
    """
    Endpoint to search for lots in the database given a string provided by the user.
    """

    def get(self, request, filters, terms=""):
        sorted_data = _upark_utilities.search_and_sort_lots(search_terms=terms, location=None, filters=filters)
        return Response(sorted_data)


class LotSearchByTermsAndLocationView(APIView):
    """
    Endpoint to search for lots in the database given a search terms string and coordinates provided by the frontend that are sorted from closest to furthest.
    """

    def get(self, request, lat, long, filters, terms=""):
        sorted_data = _upark_utilities.search_and_sort_lots(search_terms=terms, location=(lat, long), filters=filters)
        return Response(sorted_data)


class BuildingSearchView(APIView):
    """
    Endpoint to search for lots in the database given a string provided by the user.
    """
    def get(self, request, filters, terms=""):
        matches = []

        with open("../lot_data_spreadsheets/buildings.csv", "r") as building_csv:
            reader = csv.reader(building_csv)
            for row in reader:
                if terms.lower() in row[1].lower() or terms in row[2].lower():
                    matches.append({"name":row[1], "shortname":row[2], "latitude":row[3], "longitude":row[4], "building_number":row[0]})

        return Response(matches)

class ListParkingLotsAndBuildingsView(APIView):
    """
    Endpoint to get list of lots and buildings on system to facilitate autocomplete search.
    """
    def get(self, request):
        lots_and_buildings = []
        lots = ParkingLot.objects.all()
        
        for lot in lots:
            lots_and_buildings.append({"name": lot.name, "lotid": lot.lotid})

        with open("../../lot_data_spreadsheets/buildings.csv", "r") as building_csv:
            reader = csv.reader(building_csv)
            for row in reader:
                lots_and_buildings.append({"name":row[1], "shortname":row[2], "latitude":row[3], "longitude":row[4], "isbuilding": True})

        return Response(lots_and_buildings)

#---------------------AUTHENTICATION API Endpoints (secured cookie)-----------------------#
from rest_framework_simplejwt.views import TokenRefreshView, TokenObtainPairView
from rest_framework_simplejwt.serializers import TokenRefreshSerializer
from rest_framework_simplejwt.exceptions import InvalidToken

class CookieTokenRefreshSerializer(TokenRefreshSerializer):
    refresh = None
    def validate(self, attrs):
        """Get refresh token from HttpOnly Cookie and validate if it is valid"""
        attrs['refresh'] = self.context['request'].COOKIES.get('refresh_token')
        if attrs['refresh']:
            return super().validate(attrs)
        else:
            raise InvalidToken('No valid token found in cookie \'refresh_token\'')


class CookieTokenObtainPairView(TokenObtainPairView):
    def finalize_response(self, request, response, *args, **kwargs):
        if response.data.get('refresh'):
            cookie_max_age = 3600 * 24 * 2 #2 days
            response.set_cookie('refresh_token', response.data['refresh'], max_age=cookie_max_age,httponly=True,
                                samesite='strict', secure=settings.SECURE_OR_NOT)
            del response.data['refresh']
        return super().finalize_response(request, response, *args, **kwargs)

class CookieTokenRefreshView(TokenRefreshView):
    serializer_class = CookieTokenRefreshSerializer
    def finalize_response(self, request, response, *args, **kwargs):
        if response.data.get('refresh'):
            cookie_max_age = 3600 * 24 * 2 # 2 days
            response.set_cookie('refresh_token', response.data['refresh'], max_age=cookie_max_age, httponly=True )
            del response.data['refresh']
        return super().finalize_response(request, response, *args, **kwargs)


class CookieBadTokenView(APIView):
    permission_classes = [IsAuthenticated]
    def get(self, request):
        response = HttpResponse("Retrieved bad token")
        if 'refresh_token' in request.COOKIES:
            response.set_cookie('refresh_token', "verybadtokenhahahahfdhnsuifdsufhdshf", httponly=True, samesite='strict', secure=settings.SECURE_OR_NOT)
        return response



