import torch
from torch import nn
import numpy as np
import pandas as pd
from datetime import datetime
import os.path
import threading
import pytomlpp as toml
import mysql.connector
import server.settings

"""
This library is used for anything to do with lot fullness
predictions. IE: collecting data for predictions and 
running forward passes through the neural network to get
and cache new predictions.
"""


def _connect_to_db():
    """
    Connects to the database in the settings.py file with
    the supplied login information. Then returns a connection
    object.
    """
    db_settings = server.settings.DATABASES['default']
    connection = mysql.connector.connect(host='localhost',
                                         user=db_settings['USER'],
                                         passwd=db_settings['PASSWORD'],
                                         database=db_settings['NAME'])
    return connection


def _db_query(connection, query):
    """
    Executes the given query on the database with given connection. 
    Returns the result.
    """
    cursor = connection.cursor()
    cursor.execute(query)
    result = cursor.fetchall()
    cursor.close()
    return result

def _select_query(connection, table, what, where=None):
    """
    Returns a SQL select query with the given attributes/conditions.
    """
    if where:
        return f'select {what} from {table} where {where};'
    else:
        return f'select {what} from {table};'

class PredictNet(nn.Module):
    def __init__(self, num_features, hidden_dims, in_steps, out_steps, num_layers=1, dropout=0.0, device='cpu'):
        super().__init__()
        self.device=device
        self.num_layers=num_layers
        self.lstm = nn.LSTM(input_size=num_features, num_layers=self.num_layers, hidden_size=hidden_dims, batch_first=True, dropout=0.0, device=device)
        self.bnorm = nn.BatchNorm1d(hidden_dims,device=device)
        self.linear = nn.Linear(in_features=hidden_dims, out_features=out_steps, device=device)
        self.hidden_dims = hidden_dims
        self.in_steps = in_steps
        self.out_steps = out_steps
        self.num_features = num_features
        
    def forward(self, X):
        N = len(X)
        hidden = torch.randn((self.num_layers, N, self.hidden_dims), device=self.device)
        c0 = torch.zeros((self.num_layers, N, self.hidden_dims), device=self.device)
        _, (hn, cn) = self.lstm(X, (hidden, c0))
        assert hn.shape == (self.num_layers, N, self.hidden_dims)
        out = hn[self.num_layers-1].view((1,N,self.hidden_dims))
        scores = self.linear(out)
        assert scores.shape == (1, N, self.out_steps), f'Expected ({1}, {N}, {self.out_steps}), Got {scores.shape}'
        return scores.view((N, self.out_steps))


class UpdateType:
    """
    The possible types of updates for predictions. EX: HALFHOUR is predictions based on half hour increments and
    also represents saving data every half of an hour.
    """
    HALFHOUR = 'halfhour'
    DAY = 'day'
    MINUTE = 'minute'


def _get_prediction_size(type=UpdateType.HALFHOUR):
    """
    returns the size of the output prediction for the given type. 
    """
    if type == UpdateType.HALFHOUR:
        return 6
    else:
        raise NotImplementedError("Only half hour predictions are implemented.")


def _get_prediction_filename(type=UpdateType.HALFHOUR):
    """
    returns the filename where current predictions are stored. Does not guarantee the file exists.
    """
    return f'data/predictions_{type}.toml'


def _save_prediction(lot_id, prediction, type=UpdateType.HALFHOUR):
    """
    saves the most recent prediction to a file.
    """
    #print(os.path.abspath(__file__))
    filename = _get_prediction_filename(type)
    if os.path.exists(filename):
        predictions = toml.load(filename)
        predictions[str(lot_id)] = prediction
        toml.dump(predictions, filename)
    else:
        file = open(filename, 'x')
        predictions = {str(lot_id) : prediction}
        toml.dump(predictions, file)


def _get_saved_prediction(lot_id, type=UpdateType.HALFHOUR):
    """
    returns the saved most recent prediction for the given lot.
    """
    filename = _get_prediction_filename(type)
    if os.path.exists(filename):
        file = open(filename)
        predictions = toml.load(file)
        if str(lot_id) in predictions:
            return predictions[str(lot_id)]
    return None


def _get_settings():
    """
    returns settings for the neural network. device, datatype, etc.
    """
    if torch.cuda.is_available():
        return {'device' : 'cuda'}
    else:
        # print('WARNING: CUDA not available. Prediction performance may suffer.')
        return {'device' : 'cpu'}


def get_capacity(lot_id):
    """
    returns the capacity of a lot given its lot number.
    """
    capacity = 0
    connection = _connect_to_db()
    result = _db_query(connection,
                       _select_query(connection, 
                                     table='AcceptedPass', 
                                     what='allottedStalls', 
                                     where=f'lotID={lot_id}'))

    for allotted in result:
        capacity += allotted[0]

    assert capacity >= 0
    connection.close()
    return capacity


def get_latest_prediction(lot_id, type=UpdateType.HALFHOUR):
    """
    gets the latest calculated prediction without running the neural network.
    """
    prediction = _get_saved_prediction(lot_id, type)

    if not prediction:
        prediction = _predict_from_file(lot_id, type)
        _save_prediction(lot_id, prediction)
    
    assert len(prediction) == _get_prediction_size(type)
    return prediction


def _next_ts(type=UpdateType.HALFHOUR):
    """
    gets the timestamp for the next half hour interval
    """
    ts = datetime.now().timestamp()
    if type == UpdateType.HALFHOUR:
        # 1800 seconds in half an hour
        next_ts = 1800 * (ts// 1800) + 1800
    elif type == UpdateType.DAY:
        next_ts = 86400 * (ts // 86400) + 86400
    elif type == UpdateType.MINUTE:
        next_ts = 60 * (ts // 60) + 60
    else:
        raise TypeError("Invalid type requested.")
    assert next_ts > ts
    return next_ts


def _update(type=UpdateType.HALFHOUR):
    """
    updates the prediction data on disk and then uses the new data to update fullness predictions.
    """
    print('Running prediction update.')
    connection = _connect_to_db()
    lot_set = _db_query(connection,
                       _select_query(connection, 
                                     table='ParkingLot', 
                                     what='lotID, usedStalls'))
    lotID = 0
    usedStalls = 1

    for lot_info in lot_set:
        lot_id = lot_info[lotID]
        capacity = get_capacity(lot_id)
        occupancy = lot_info[usedStalls]
        assert capacity >= occupancy, f"Capacity or Occupancy values are incorrect. Occupancy = {occupancy}, Capacity = {capacity}"
        dow = datetime.now().isoweekday()
        if capacity == 0:
            continue
        update_dow_avg(lot_id, dow, capacity, occupancy)
        dow_avg = get_dow_avg(lot_id, dow)
        assert 0.0 <= dow_avg <= 1.0, f"Incorrect day of week average found. dow_avg = {dow_avg}"
        save_data(lot_id, datetime.now().timestamp(), capacity, occupancy, dow_avg)
        prediction = _predict_from_file(lot_id)
        _save_prediction(lot_id, prediction)

    connection.close()
    print('Finished running prediction update.')
    # schedule_update(type)


def schedule_update(type=UpdateType.HALFHOUR):
    """
    sets up a thread to run at the next half hour time interval which updates all the predictions.
    """
    ts = datetime.now().timestamp()
    ts_delta = _next_ts(type) - ts
    threading.Timer(ts_delta, _update, args=[type]).start()


def _ema(prev_avg, new_val, multiplier):
    """ Given the previous exponential moving average (EMA), the new value, and a multiplier, returns
    the new EMA.
    """
    return prev_avg + multiplier * (new_val - prev_avg)


def _num_samples_to_multiplier(num_samples):
    """ Given the number of samples you'd like to average over, returns 
    the multiplier for use in EMA.
    """
    return 2 / (num_samples + 1)


def _get_prediction_data_filename(lot_id, type=UpdateType.HALFHOUR):
    """
    the location on disk of the current lot data.
    """
    return f'data/lot{lot_id}_{type}_data.csv'


def _load_data(lot_id, type=UpdateType.HALFHOUR):
    """
    load data from disk into a pandas dataframe.
    """
    filename = _get_prediction_data_filename(lot_id, type)
    data = pd.read_csv(filename) if os.path.exists(filename) else pd.DataFrame(columns=['Timestamp', 'Capacity', 'Occupancy', 'DOWAvg'])
    return data


def _get_dow_filename():
    """
    where the day of week average data is store on disk.
    """
    return 'data/dow_avgs.csv'


def get_dow_avg(lot_id, dow):
    """
    gets the dat of week average from disk and returns it.
    """
    filename = _get_dow_filename()

    if not os.path.exists(filename):
        return None

    data = pd.read_csv(filename)
    lot_data = data[data['LotID'] == lot_id]
    return lot_data[str(dow)].iloc[0]


def _replace_dow_avg_row(data, lot_id, dow, percent_full):
    """
    creates a new dataframe where the lot's row is updated based on the new data
    """
    

    new_data = data.copy()
    if len(data) != 0 and (data['LotID'] == lot_id).any():
        row = data[data['LotID'] == lot_id]
        old_avg = row[str(dow)].iloc[0]
        row_data = [lot_id, 
                    row['1'].iloc[0], 
                    row['2'].iloc[0], 
                    row['3'].iloc[0], 
                    row['4'].iloc[0], 
                    row['5'].iloc[0], 
                    row['6'].iloc[0], 
                    row['7'].iloc[0]]
        # remember a year
        if old_avg != 0:
            dow_avg = _ema(old_avg, percent_full, _num_samples_to_multiplier(365 * 48))
        else:
            dow_avg = percent_full
        row_data[dow] = dow_avg
        new_data[new_data['LotID'] == lot_id] = row_data
        assert len(data) == len(new_data)
    else:
        row_data = pd.DataFrame()
        row_data['LotID'] = [lot_id]
        row_data['1'] = [0.]
        row_data['2'] = [0.]
        row_data['3'] = [0.]
        row_data['4'] = [0.]
        row_data['5'] = [0.]
        row_data['6'] = [0.]
        row_data['7'] = [0.]
        row_data[str(dow)] = [percent_full]
        new_data = pd.concat(objs=[new_data, row_data], ignore_index=True)
        assert len(data) + 1 == len(new_data) 
    
    return new_data


def update_dow_avg(lot_id, dow, capacity, occupancy):
    """
    updates the day of week percent full avg on disk for a given lot on a given day of the week (iso format).
    """
    filename = _get_dow_filename()
    percent_full = occupancy / capacity
    data = pd.read_csv(filename) if os.path.exists(filename) else pd.DataFrame()
    data = _replace_dow_avg_row(data, lot_id, dow, percent_full)
    data.to_csv(filename, index=False)


def _add_data(old_data, lot_id, timestamp, capacity, occupancy, dow_avg, type=UpdateType.HALFHOUR):
    """
    returns a new dataframe where the new data is added to the end of the given dataframe
    """
    new_row = pd.DataFrame()
    new_row['Timestamp'] =  [timestamp]
    new_row['Capacity'] = [capacity]
    new_row['Occupancy'] = [occupancy]
    new_row['DOWAvg'] = [dow_avg]
    new_data = pd.concat(objs=[old_data, new_row], ignore_index=True)
    return new_data


def save_data(lot_id, timestamp, capacity, occupancy, dow_avg, type=UpdateType.HALFHOUR):
    """
    save the given data to disk
    """
    old_data = _load_data(lot_id, type)
    filename = _get_prediction_data_filename(lot_id, type)
    new_data = _add_data(old_data, lot_id, timestamp, capacity, occupancy, dow_avg, type)
    new_data.to_csv(filename, index=False)


def _init_model(type=UpdateType.HALFHOUR):
    """
    set up the prediction neural network
    """
    num_features = 7
    hidden_dims = 60
    num_layers = 2
    dropout = 0.0
    in_steps = 12
    out_steps = _get_prediction_size(type)
    return PredictNet(num_features, hidden_dims, in_steps, out_steps, num_layers, dropout, **_get_settings())


def _preprocess_data(dataframe, type=UpdateType.HALFHOUR):
    """
    takes the raw data as stored on disk and formats it for use in the neural network. Recommended to only do this on data going into the network.
    DON'T STORE PREPROCESS DATA.
    """
    if type == UpdateType.HALFHOUR or type == UpdateType.MINUTE:
        new_data = pd.DataFrame()
        # new_data['Timestamp'] = [(ts -  datetime.fromisoformat('2010-01-01').timestamp()) / timedelta(days=365).total_seconds() for ts in dataframe['Timestamp']]
        new_data['DayOfWeek'] = [datetime.fromtimestamp(ts).isoweekday() / 7 for ts in dataframe['Timestamp']]
        new_data['Month'] = [datetime.fromtimestamp(ts).month / 12 for ts in dataframe['Timestamp']]
        new_data['Day'] = [datetime.fromtimestamp(ts).day / 31 for ts in dataframe['Timestamp']]
        new_data['Hour'] = [datetime.fromtimestamp(ts).hour / 24 for ts in dataframe['Timestamp']]
        new_data['DOWAvg'] = dataframe['DOWAvg'].to_numpy()
        new_data['Capacity'] = [r.Capacity / 1000 for r in dataframe[['Capacity']].itertuples()]
        new_data['PercentFull'] = [r.Occupancy / r.Capacity for r in dataframe[['Capacity', 'Occupancy']].itertuples()]
    elif type == UpdateType.DAY:
        raise NotImplementedError("Day predictions are not implemented yet.")
    else:
        raise TypeError("Invalid type given.")
    return new_data


def _get_default_prediction(lot_id, type=UpdateType.HALFHOUR):
    """
    returns some default prediction for a given lot if there is not enough data available for an accurate prediction
    """
    if type == UpdateType.HALFHOUR or type == UpdateType.MINUTE:
        historical_data = [ 0.025318, 0.03279, 0.033176, 0.05213, 
                            0.060114, 0.082262, 0.086682, 0.10195, 0.110376, 0.139652, 
                            0.152912, 0.164962, 0.211438, 0.221838, 0.277296, 0.370884, 
                            0.51993, 0.613518, 0.759098, 0.856152, 0.897746, 0.92201, 
                            0.93241, 0.911612, 0.824956, 0.745234, 0.665512, 0.571924, 
                            0.561526, 0.495668, 0.429402, 0.366294, 0.329198, 0.272586, 
                            0.238992, 0.201798, 0.149966, 0.1241, 0.103614, 0.07778, 
                            0.06353, 0.038514, 0.044752, 0.01806, 0.013276, 0.014968, 
                            0.020398, 0.01296]
        hour = datetime.now().hour + 2
        min = datetime.now().minute
        idx = hour * 2 + (1 if min > 30 else 0)
        if idx + 6 >= len(historical_data):
            diff = (idx + 6) - len(historical_data)
            return historical_data[idx:] + historical_data[:diff]
        else:
            return historical_data[idx:idx+6]
        #return [get_dow_avg(lot_id, datetime.now().isoweekday())] * _get_prediction_size(type)
    elif type == UpdateType.DAY:
        raise NotImplementedError('Day predictions are not implemented.')
    else:
        raise TypeError("Invalid type given.")


def _get_model_filename(type=UpdateType.HALFHOUR):
    return f'library/{type}_predict_model.pt'


def _predict(data, type=UpdateType.HALFHOUR):
    """
    given a raw (not pre-processed) dataframe of the neural networks input size, returns a neural network prediction.
    """
    
    model = _init_model()
    model.load_state_dict(torch.load(_get_model_filename(), map_location=torch.device(_get_settings()['device'])))
    model.eval()
    prediction = model(data.unsqueeze(dim=0)).squeeze().tolist()
    assert len(prediction) == _get_prediction_size(type), f"Got len(prediction)={len(prediction)} but expected size {_get_prediction_size(type)}."
    return [1 if e > 1 else e for e in [0 if e < 0 else e for e in prediction]]


def _predict_from_file(lot_id, type=UpdateType.HALFHOUR):
    """
    Reads from disk the data needed to predict lot fullness. Then calculates and returns that prediction.
    """
    data = _load_data(lot_id, type)
    

    if type == UpdateType.HALFHOUR or type==UpdateType.MINUTE:
        if len(data) < 12:
            return _get_default_prediction(lot_id, type)
        data = data.iloc[-12:] # last 12 rows
    elif type == UpdateType.DAY:
        raise NotImplementedError("Day predictions are not implemented.")
    else:
        raise TypeError("Invalid type given.")
    
    data = _preprocess_data(data, type)
    data = torch.tensor(data.to_numpy(dtype=np.float32), **_get_settings())
    prediction = _predict(data)
    return prediction
