import React, { useState, useEffect, useMemo, useCallback } from "react";
import { MapContainer, TileLayer, Marker,  } from "react-leaflet";
import "leaflet/dist/leaflet.css";
import "./AdminMap.css";
import { fetchLotsLayer } from "./AdminMapUtils";
import axios from "axios";
import {updateLotData, updateMapData, updateLotAndBuildingData, updateSensorData, updateEnexData} from "../../../redux/mapSlice";
import { useDispatch, useSelector } from "react-redux";
import {useHistory, useLocation, matchPath } from "react-router";
import LotsLayer from "./LotsLayer";
import PickLocationMarker from "./PickLocationMarker";


//Imports for Entrance/Exit labeling
import L from 'leaflet';
import icon from 'leaflet/dist/images/marker-icon.png';
import iconShadow from 'leaflet/dist/images/marker-shadow.png';

// Set up the icon for entrance exit labeling on the admin map
let EnEx_NoSensor_Icon = new L.icon({
    iconUrl: require('./enex_nosensor.png'),
    iconSize: [15, 15],
    shadowUrl: null,
    shadowSize: null,
    iconAnchor:null,
    popupAnchor:null,
    //className: 'leaflet-div-icon'

});

// Set up the icon for entrance exit labeling on the admin map
let EnEx_WithSensor_Icon = new L.icon({
  iconUrl: require('./enex_plus_sensor_blue.png'),
  iconSize: [15, 15],
  shadowUrl: null,
  shadowSize: null,
  iconAnchor:null,
  popupAnchor:null,
  //className: 'leaflet-div-icon'

});

// Set up the icon for general parking lot marking
let General_Lot_Icon = new L.icon({
  iconUrl: require('./parking-p-sign.png'),
  iconSize: [20, 20],
  shadowUrl: null,
  shadowSize: null,
  iconAnchor:null,
  popupAnchor:null,
  //className: 'leaflet-div-icon'

});

export const useGetEnexData = () => {
  const dispatch = useDispatch();
  const currentURL = useLocation();
  const match = matchPath(currentURL.pathname, {
    path: "/admin/dashboard/parkinglots/edit/:id",
    exact: true,
  });
  const selectedLotId = match?.params?.id

  const getEnexData = useCallback(() => {
    const EnexUrl = selectedLotId === undefined ? `${process.env.REACT_APP_API_URL}/api/enex/`: `${process.env.REACT_APP_API_URL}/api/lot/${selectedLotId}/entranceexits`;
    const getEnexConfig = {
      url: EnexUrl,
      method: "GET",
      headers: {
        "Content-Type": "application/json",
      },
    }
    axios(getEnexConfig)
      .then(response => {
        const data = Array.isArray(response.data) ? response.data:[response.data];
        dispatch(updateEnexData(data));
        //added to keep having global enex list
        if (selectedLotId === undefined){
          dispatch(updateMapData({globalEnexData:response.data}));
        }
      })
      .catch(error=>{
        console.error("error enex get", error);
      })
  },[selectedLotId])

    return getEnexData
}

// Get the latitude and longitude of Entrance/Exits to place on the admin map
const useEnExPositions = () => {
  //const [enExArray, setenExArray] = useState([]);
  const enexArray = useSelector((state) => state.map.enexData);
  const getEnexData = useGetEnexData();
  const currentURL = useLocation();
  const match = matchPath(currentURL.pathname, {
    path: "/admin/dashboard/parkinglots/edit/:id",
    exact: true,
  });

  const selectedLotId = match?.params?.id

  useEffect(() => {
    getEnexData();
  }, [getEnexData, selectedLotId])

    return enexArray
}

export const useGetSensorData = () => {
  const dispatch = useDispatch();
  const getSensorData = useCallback(() => {
    const sensorURL =`${process.env.REACT_APP_API_URL}/api/sensor/`;
    const getSensorConfig = {
      url: sensorURL,
      method: "GET",
      headers: {
        "Content-Type": "application/json",
      },
    }
    axios(getSensorConfig)
      .then(response => {
        dispatch(updateSensorData(response.data))
      })
      .catch(error=>{
        console.error(error);
      })

  },[])
    return getSensorData
}

// Get an array of all sensors in the system
export const useSensorData = () => {
  const sensorArray = useSelector((state) => state.map.sensorData);
  const getSensorData = useGetSensorData();

  useEffect(() => {
    getSensorData();
  },[getSensorData])

  return sensorArray
}

// Get an array of all parking lots in the system
export const useParkingLotData = () => {

  const parkingLotArray = useSelector((state) => state.map.lotData);
  const parkingLotURL = `${process.env.REACT_APP_API_URL}/api/lot/`;
  const dispatch = useDispatch();
  const [refreshState, setRefreshState] = useState(0);

  const getParkingLotConfig = {
    url: parkingLotURL,
    method: "GET",
    headers: {
      "Content-Type": "application/json",
    },
  }
  useEffect(() => {
    axios(getParkingLotConfig)
      .then(response => {
        dispatch(updateLotData(response.data));
      })
      .catch(error=>{
        console.error(error);
      })
    }, [refreshState])

    const getParkingLotData = () => {
      setRefreshState(refreshState + 1);
    }

    return [parkingLotArray,getParkingLotData]
}

// Get an array of all parking lots and buildings in the system
export const useParkingLotAndBuildingData = () => {

  const lotAndBuildingArray = useSelector((state) => state.map.lotAndBuildingData);
  const lotsAndBuildingURL = `${process.env.REACT_APP_API_URL}/api/lotsandbuildings/`;
  const dispatch = useDispatch();
  const [refreshState, setRefreshState] = useState(0);

  const getLotAndBuidingConfig = {
    url: lotsAndBuildingURL,
    method: "GET",
    headers: {
      "Content-Type": "application/json",
    },
  }
  useEffect(() => {
    axios(getLotAndBuidingConfig)
      .then(response => {
        dispatch(updateLotAndBuildingData(response.data));
      })
      .catch(error=>{
        console.error(error);
      })
    }, [refreshState])

    const getParkingLotAndBuildingData = () => {
      setRefreshState(refreshState + 1);
    }

    return [lotAndBuildingArray,getParkingLotAndBuildingData]
}

// This function returns a boolean saying whether the entrance exit in question has a sensor assigned to it
//const isEnexSensored = (sensorData, enex) => Boolean(sensorData.find(sensor => sensor.enid === enex.enid));

// This function adds a hasSensor field to entrance/exit data to show whether an enex has a sensor assigned to it
const enexDataWithSensorData = (enexData, sensorData, parkingLotData) => {
  return enexData.map(enex => {
    const sensor = sensorData.find(sensor => sensor.enid === enex.enid);
    const hasSensor = Boolean(sensor);
    const lot = parkingLotData.filter(lot => lot.lotid === enex.lotid);
    const defaultedSensor = sensor || {}
    const defaultedLot = lot || {}
    return {...enex, ...defaultedSensor, ...defaultedLot, hasSensor}
  }
)}

const hasGeoJSONData = (features = [], lot) => Boolean(features.find(lotWithGeo => parseInt(lotWithGeo.properties.PLOT) === lot.lotid));

const toNonGeoJSONParkingLots = (mapData, parkingLotData) => {
  return parkingLotData.filter(lot => !hasGeoJSONData(mapData?.features, lot))
}

function AdminMap() {
  const [mapData, setMapData] = useState();
  const lotStallsRemaining={};
  const lotPercents={};
  const [lotPercentsFinished, setLotPercentsFinished] = useState(false);
  const [lotStallsRemainingFinished, setLotStallsRemainingFinished] = useState(false);
  const dispatch = useDispatch();
  const enexData = useEnExPositions();
  const sensorData = useSensorData();
  const [parkingLotData] = useParkingLotData();
  let history = useHistory();


  const enexDataWithSensor = useMemo(() => enexDataWithSensorData(enexData, sensorData, parkingLotData), [enexData, sensorData, parkingLotData]);
  const nonGeoJSONParkingLots = useMemo(() => toNonGeoJSONParkingLots(mapData,parkingLotData), [mapData, parkingLotData]);

  useEffect(() => {
    if (!mapData){
      fetchLotsLayer()
        .then((response) => response.json())
        .then((result) => {
          setMapData(result);
        })
        .catch((error) => console.error(error));
    }

    //get percentFull for all lots
    if (mapData){
      for (let feature of mapData.features){
        let plot = parseInt(feature.properties.PLOT);
        const percentUrl = `${process.env.REACT_APP_API_URL}/api/lot/${plot}/percentfull/`;
        const getPercentConfig = {
          url: percentUrl,
          method: "GET",
          headers: {
            "Content-Type": "application/json",
          },
        }
        axios(getPercentConfig)
        .then(response => {
          lotPercents[plot] = response.data;
        })
        .catch(error=>{
          lotPercents[plot] = "nodata";
        })
        .finally(()=>{
          if (Object.keys(lotPercents).length === Object.keys(mapData.features).length){
            setLotPercentsFinished(true);
            dispatch(updateMapData({lotPercents:lotPercents}));
          }
      });
    }

      for (let feature of mapData.features){
        let plot = parseInt(feature.properties.PLOT);
        const stallsRemainingUrl = `${process.env.REACT_APP_API_URL}/api/lot/${plot}/stallsremaining/`;

        const getStallsRemainingConfig = {
              url: stallsRemainingUrl,
              method: "GET",
              headers: {
                "Content-Type": "application/json",
              },
            }

        axios(getStallsRemainingConfig)
        .then(response => {
          lotStallsRemaining[plot] = response.data; // change to lotPercents if percetage threshold is used
        })
        .catch(error=>{
          lotStallsRemaining[plot] = "nodata";
        })
        .finally(()=>{
          if (Object.keys(lotStallsRemaining).length === Object.keys(mapData.features).length){
            setLotStallsRemainingFinished(true);
            dispatch(updateMapData({lotStallsRemaining:lotStallsRemaining})); // change to lotPercents if percentage threshold is used
          }
        })
      }
    }
  }, [mapData]);

  //mapbox://styles/bluqzy/ckywcvgqx001e15obhw6nfyqz
  //pk.eyJ1IjoiYmx1cXp5IiwiYSI6ImNrdXl6aHBrODc3OGIycG56cG0yZmNwanEifQ.sayISMyWW1fvtwbesghNvw
  // https://api.mapbox.com/styles/v1/bluqzy/ckywcvgqx001e15obhw6nfyqz/tiles/{tilesize}/{z}/{x}/{y}{@2x}?access_token=pk.eyJ1IjoiYmx1cXp5IiwiYSI6ImNreXdkNWcwYzA3Z2UydG85aHI2OGlmbHEifQ.VhCvUs5GZh_43yNdmWMNkQ
  //https://api.mapbox.com/styles/v1/mapbox/satellite-v9/tiles/512/10/197/384%7B@2x%7D?access_token=pk.eyJ1IjoiYmx1cXp5IiwiYSI6ImNrdXl6aHBrODc3OGIycG56cG0yZmNwanEifQ.sayISMyWW1fvtwbesghNvw
  return (
    <div className="map-wrapper z-[1000]">
      {/* <MapPin/> */}
      <MapContainer
      //   whenReady={(map) => {
      //     map.target.on("click", function (e) {
      //       history.push(`/admin/dashboard/`);
      //     });
      // }}
      center={[40.76, -111.891]}
      zoom={13}
      scrollWheelZoom={true}
      zoomControl={false}>
        <TileLayer
          attribution='&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
          url="https://api.mapbox.com/styles/v1/bluqzy/ckywcvgqx001e15obhw6nfyqz/tiles/256/{z}/{x}/{y}@2x?access_token=pk.eyJ1IjoiYmx1cXp5IiwiYSI6ImNrdXl6aHBrODc3OGIycG56cG0yZmNwanEifQ.sayISMyWW1fvtwbesghNvw"
          />
        <PickLocationMarker/>
        {
          // Add Entrance and Exit icons
          enexDataWithSensor ? enexDataWithSensor.map(enex => (
          <Marker
            eventHandlers={{
              click: (e) => {
                if (!enex.hasSensor)
                {
                  history.push(`/admin/dashboard/parkinglots/edit/${parseInt(enex.lotid)}`);

                }
                else
                {
                  history.push(`/admin/dashboard/sensors/edit/${parseInt(enex.sensorid)}`);
                }
              }
            }}
            key={enex.enid}
            icon={enex.hasSensor ? EnEx_WithSensor_Icon : EnEx_NoSensor_Icon}
            position={[enex.latitude,enex.longitude]}/>)):null
        }
        {/* Add GeoJSON polygons */}
        {!!(mapData && lotPercentsFinished && lotStallsRemainingFinished) && <LotsLayer mapData={mapData} lotPercentsFinished={lotPercentsFinished}/>}
        {/* Add general pinning for lots without GeoJSON */}
        {
          (nonGeoJSONParkingLots) ? nonGeoJSONParkingLots.map(lot => (
            <Marker
              eventHandlers={{
                click: (e) => {
                  //e.preventDefault()
                  history.push(`/admin/dashboard/parkinglots/edit/${parseInt(lot.lotid)}`);
                }
              }}
              key={lot.lotid}
              icon={General_Lot_Icon}
              position={[lot.latitude,lot.longitude]}/>
          )) : null
        }

      </MapContainer>
    </div>
  );
}

export default AdminMap;
