import axios from "axios";
import React, { useState, useEffect } from "react";
import { useHistory, useLocation } from "react-router-dom";
import { useDispatch, useSelector } from "react-redux";
import { LocationMarkerIcon, XIcon } from "@heroicons/react/solid";
import { updateMapData } from "../../../redux/mapSlice";
import { useGetEnexData } from "../adminmap/AdminMap";

export default function EnexPost() {
  const [apiError, setApiError] = useState(false);
  const [responseSuccess, setResponseSuccess] = useState(false);
  const getEnexData = useGetEnexData();
  const enexListURL = `${process.env.REACT_APP_API_URL}/api/enex/`;
  const location = useLocation();
  const history = useHistory();
  const dispatch = useDispatch();
  const adminPickingOnMapState = useSelector((state) => state.map.adminPickingOnMapState);
  const adminPickingOnMapLocation = useSelector((state) => state.map.adminPickingOnMapLocation);
  const [enexInfo, setEnexInfo] = useState(() => {
    //If the router specifies some pre population data
    if (location.state && location.state.defaultAddEnex)
      return {
        lotid: location.state.defaultAddEnex.lotid ? location.state.defaultAddEnex.lotid : "",
        longitude: location.state.defaultAddEnex.longitude ? location.state.defaultAddEnex.longitude : "",
        latitude: location.state.defaultAddEnex.latitude ? location.state.defaultAddEnex.latitude : "",
      };
    return {
      lotid: "",
      longitude: "",
      latitude: "",
    };
  });

  //
  // useEffect to update enexInfo after picking a location on Map
  //
  useEffect(() => {
    if (adminPickingOnMapState === "pick" && adminPickingOnMapLocation) {
      setEnexInfo({
        ...enexInfo,
        ["latitude"]: adminPickingOnMapLocation.lat.toFixed(6),
        ["longitude"]: adminPickingOnMapLocation.lng.toFixed(6),
      });
    }
  }, [adminPickingOnMapState, adminPickingOnMapLocation]);
  const handleChange = (event) => {
    setEnexInfo({ ...enexInfo, [event.target.name]: event.target.value });
    setResponseSuccess(false);
  };
  const handleSubmit = (e) => {
    e.preventDefault();
    const postConfig = {
      url: enexListURL,
      method: "POST",
      headers: {
        Authorization: `Bearer ${localStorage.getItem("token")}`,
        "Content-Type": "application/json",
      },
      data: { ...enexInfo },
      withCredentials: true,
    };

    axios(postConfig)
      .then(function (response) {
        // Reset Form
        setEnexInfo({
          lotid: "",
          longitude: "",
          latitude: "",
        });
        setApiError(false);
        setResponseSuccess(true);
        dispatch(updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined }));
        getEnexData();
      })
      .catch((error) => {
        setApiError(true);
        console.error(error);
        dispatch(updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined }));
      });
  };
  const handleGoBack = (e) => {
    e.preventDefault();

    if (location.key) {
      dispatch(updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined }));
      history.goBack();
    }
  };

  const handlePickOnMap = (e) => {
    e.preventDefault();
    dispatch(updateMapData({ adminPickingOnMapState: "pick" }));
  };

  // USE effect to reset picking on map states when first load
  // and reset the picking location state on unmount
  useEffect(() => {
    dispatch(updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined }));
    return () => {
      dispatch(updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined }));
    };
  }, []);

  //   Render
  return (
    <>
      {enexInfo && (
        <div className="flex flex-col justify-between w-full lot-post">
          <div className="max-w-sm px-1 my-2 rounded-xl">
            {/* Add a Goback button here */}
            <div className="flex flex-row w-full">
              <button
                disabled={!location.key}
                onClick={(e) => handleGoBack(e)}
                className="flex flex-row justify-center p-2 transition-all duration-700 transform bg-white border rounded-full shadow-lg w-9 h-9 group hover:w-28 hover:justify-around hover:bg-amaranth-500 border-amaranth-300/20 text-slate-700"
              >
                <i className="flex self-center text-lg fa-solid fa-chevron-left text-amaranth-400 group-hover:text-white"></i>
                <span className="self-center hidden text-lg text-white truncate transition-all duration group-hover:flex">
                  Go Back
                </span>
              </button>
            </div>
            <form onSubmit={handleSubmit}>
              <div className="flex flex-col justify-between gap-2 ">
                <div className="flex flex-col w-full rounded-md">
                  <div>
                    <label htmlFor="enid" className="block mb-1 font-semibold text-gray-600 uppercase">
                      Lot ID
                    </label>
                    <input
                      type="text"
                      id="lotid"
                      name="lotid"
                      className="w-full px-2 py-2 text-sm transition-all duration-150 ease-linear bg-white border-0 rounded shadow placeholder-blueGray-300 text-blueGray-600 focus:outline-none focus:ring"
                      onChange={handleChange}
                      value={enexInfo.lotid}
                      required
                    />
                  </div>
                  <div>
                    <button onClick={(e) => handlePickOnMap(e)} className="flex flex-row mt-0.5 mb-1 flex-nowrap p-2 bg-slate-100 rounded-lg">
                      <LocationMarkerIcon className="w-5 h-5 mr-2"></LocationMarkerIcon>
                      {adminPickingOnMapState === undefined && <span className="">Pick on Map</span>}
                      {adminPickingOnMapState === "pick" && <span className="">Drag the marker to pick location</span>}
                    </button>
                    {adminPickingOnMapState === "pick" && (
                      <button
                        onClick={(e) => {
                          dispatch(
                            updateMapData({ adminPickingOnMapLocation: undefined, adminPickingOnMapState: undefined })
                          );
                        }}
                        className="flex flex-row mt-0.5 mb-1 flex-nowrap p-2 bg-gray-200 rounded-lg"
                      >
                        <XIcon className="w-5 h-5 mr-1 "></XIcon>
                        <span className="">Cancel</span>
                      </button>
                    )}
                    <label htmlFor="latitude" className="block mb-1 font-semibold text-gray-600 uppercase">
                      Latitude
                    </label>
                    <input
                      type="text"
                      id="latitude"
                      name="latitude"
                      className="w-full px-2 py-2 text-sm transition-all duration-150 ease-linear bg-white border-0 rounded shadow placeholder-blueGray-300 text-blueGray-600 focus:outline-none focus:ring"
                      onChange={handleChange}
                      value={enexInfo.latitude}
                      required
                    />
                  </div>
                  <div>
                    <label htmlFor="latitude" className="block mb-1 font-semibold text-gray-600 uppercase">
                      Longitude
                    </label>
                    <input
                      type="text"
                      id="longitude"
                      name="longitude"
                      className="w-full px-2 py-2 text-sm transition-all duration-150 ease-linear bg-white border-0 rounded shadow placeholder-blueGray-300 text-blueGray-600 focus:outline-none focus:ring"
                      onChange={handleChange}
                      value={enexInfo.longitude}
                      required
                    />
                  </div>
                  <div className="flex-row block gap-2">
                    <button
                      type="submit"
                      className="w-full py-2 mt-4 text-lg font-semibold tracking-wider text-white rounded-md bg-gradient-to-tr from-amaranth-500 to-amaranth-400 hover:from-amaranth-500 hover:to-amaranth-300 focus:outline-none focus:ring focus:ring-pink-500"
                    >
                      Submit
                    </button>
                  </div>
                </div>
              </div>
            </form>
          </div>
          {apiError === true && <h5>Cannot create a new Entrance/Exit</h5>}
          {responseSuccess === true && <h5>Entrance/Exit created successfully</h5>}
        </div>
      )}
    </>
  );
}
