import React, { useState, useEffect } from "react";
import { verifyAccessToken, refreshToken } from "../../../utils/auth";
import { useHistory } from "react-router";
import Popup from "./Popup";

export default function SetupForm() {
  const [systemName, setSystemName] = useState("");
  const [shortName, setShortName] = useState("");
  const [selectedFile, setSelectedFile] = useState(null);
  const [isAuth, setIsAuth] = useState();
  const history = useHistory();
  const [showSuccess, setShowSuccess] = useState(false);
  const [showFailure, setShowFailure] = useState(false);
  const [failureMessage, setFailureMessage] = useState("");

  useEffect(() => {
     //
    // Check if localstorage's token is valid still
    //
    if (localStorage.getItem("token") === null) {
      setIsAuth(false);
    } else {
      verifyAccessToken()
        .then(function (response) {
          if (response.status === 200) setIsAuth(true);
        })
        .catch((error) => {
          console.error("cached access token denied");
          setIsAuth(false);
        });
    }
    if (isAuth === false)
      refreshToken()
        .then(function (response) {
          if (response.status === 200) {
            localStorage.setItem("token", response.data.access);
            setIsAuth(true);
          }
        })
        .catch((error) => {
          console.error("cached refresh token denied");
          localStorage.removeItem("token");
          setIsAuth(false);
          history.push("/admin/login");
        });
  },[isAuth])

  const handleSubmit = (e) => {
    e.preventDefault();

    //
    // Send logo to upload endpoint
    //
    let data = new FormData();

    if(!selectedFile.name.endsWith('png') && !selectedFile.name.endsWith('PNG')){
      setFailureMessage("Invalid image file. PNG files only.");
      setShowFailure(true);
      return;
    }


    let newFile = new File([selectedFile], "profile.png", {
      type: selectedFile.type,
    });
    data.append("description", "logo");
    data.append("file", newFile);
    data.append("folder", "images"); //Uploading to 'images/' folder since this is the logo part

    const send_logo = fetch(`${process.env.REACT_APP_API_URL}/api/setup/mediastorage/`, {
      method: "POST",
      headers: {
        Authorization: `Bearer ${localStorage.getItem("token")}`,
      },
      body: data,
    });

    //
    // Send logo to upload endpoint
    //
    const systemInfo = {
      systemName: systemName.trim(),
      shortName: shortName.trim(),
    };

    const send_sysInfo = fetch(`${process.env.REACT_APP_API_URL}/api/setup/systeminfo/`, {
      method: "POST",
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${localStorage.getItem("token")}`,
      },
      body: JSON.stringify(systemInfo),
    });

    //Group two promises together and reload page after they are done
    Promise.all([send_logo, send_sysInfo])
      .then(function(result){
        setShowSuccess(true);
      } )
      .catch(err => {
        console.error(err);
        setFailureMessage("Error encountered. Try again.");
        setShowFailure(true);
      });
  };

  function handleAfterSuccess(){
    setShowSuccess(false);
    window.location.replace('dashboard');
  }

  function handleAfterFailure(){
    setShowFailure(false);
    window.location.reload();
  }

  return <>
    {isAuth === true && ( <div className="relative flex flex-col items-center min-h-screen bg-indigo-300 sm:justify-center ">
    <div className="relative w-full sm:max-w-sm">
        <div className="relative w-screen max-w-sm px-10 py-8 my-2 bg-white shadow-md rounded-xl">
        {showSuccess && <Popup message="System Setup Successful!" onClick={handleAfterSuccess}></Popup>}
        {showFailure && <Popup message={failureMessage} onClick={handleAfterFailure}></Popup>}
      <form onSubmit={handleSubmit}>
          <div className="space-y-4 space-y">
            <h1 className="text-2xl font-semibold text-center text-gray-600">Initialize UPark</h1>
            <div>
              <label htmlFor="systemName" className="block mb-1 font-semibold text-gray-600">
                System Name
              </label>
              <input
                type="text"
                pattern=".*\w.*"
                id="systemName"
                className="w-full px-4 py-2 rounded-md outline-none bg-indigo-50"
                onChange={(e) => setSystemName(e.target.value)}
                required
              />
            </div>
            <div>
              <label htmlFor="systemName" className="block mb-1 font-semibold text-gray-600">
                Short Name
              </label>
              <input
                type="text"
                pattern="\s*[^\s]+\s*"
                id="shortName"
                className="w-full px-4 py-2 rounded-md outline-none bg-indigo-50"
                onChange={(e) => setShortName(e.target.value)}
                required
              />
            </div>
            <div>
              <label htmlFor="formFile" className="block mb-1 font-semibold text-gray-600">
                Upload Logo
              </label>
              <input
                className="block w-full overflow-hidden text-gray-800 bg-white border border-gray-300 rounded-md shadow-sm cursor-pointer focus:ring-blue-600 focus:outline-none focus:ring-2 focus:border-transparent"
                type="file"
                onChange={(e) => setSelectedFile(e.target.files[0])}
                id="formFile"
                required
              />
            </div>
            <button
              type="submit"
              className="w-full py-2 mt-4 text-lg font-semibold tracking-wider text-white rounded-md bg-gradient-to-tr from-purple-500 to-blue-500 hover:from-pink-400 hover:to-indigo-500 focus:outline-none focus:ring focus:ring-violet-300"
            >Submit</button>
          </div>
      </form>
        </div>
    </div>
    </div>
    )}
  </>
}

