import React, { useState, useEffect } from "react";
import { useSelector, useDispatch } from "react-redux";
import { showMenuBar, hideMenuBar } from "../../redux/menuSlice";
import { useParkingLotAndBuildingData } from "../admin/adminmap/AdminMap";
import Fuse from "fuse.js";

import { useHistory, useParams } from "react-router-dom";

function MainBar({ defaultSearchTerm }) {
  const { terms, lat, long } = useParams();
  const panelDisplay = useSelector((state) => state.lotSelect.display);
  const menuToggled = useSelector((state) => state.menuToggle.toggled);
  const dispatch = useDispatch();
  const [searchState, setSearchState] = useState("");
  const currentPos = useSelector((state) => state.map.currentLocation);
  let history = useHistory();

  //const lotArray = useSelector((state) => state.map.lotData)
  const [lotAndBuildingData] = useParkingLotAndBuildingData();

  //Fuse And Search
  const [focused, setFocused] = useState(false)
  const onFocus = () => setFocused(true)
  const onBlur = () => setFocused(false)
  const maxResults = 5;
  let fuseOptions = {
    keys: ["name", "shortname"],
    threshold: 0.2,
    minMatchCharLength: 2,
  };
  const fuse = new Fuse(lotAndBuildingData, fuseOptions);

  useEffect(() => {}, [terms]);
  useEffect(() => {
    fuse.setCollection(lotAndBuildingData);
  }, [lotAndBuildingData]);
  const searchResults = React.useMemo(() => {
    if (!searchState || focused === false) {
      return [];
    }
    let fuseResults = fuse
    .search(searchState, { limit: maxResults })
    .map((result) => ({ ...result.item }))
    .slice(0, maxResults);
    // Add a fake lot item in result that says "search for lots with text"
    fuseResults = [{
      lotid:-1,
      name: `Find lots with "${searchState}"`
    }, ...fuseResults];
    return fuseResults
  }, [fuse, maxResults, searchState, focused]);

  const handleMenuToggled = (e) => {
    e.preventDefault();
    if (menuToggled) dispatch(hideMenuBar());
    else dispatch(showMenuBar());
  };

  const handleOnSelect = (item) => {
    //If item.lotid = -1 -> search term:
    if (item.lotid===-1){
      if (currentPos != null) history.push(`/map/search/${searchState}/${currentPos.lat}/${currentPos.lng}`);
      else history.push(`/map/search/${searchState}`);
    }
    else if (item.isbuilding) {
      history.push(`/map/search/*/${item.latitude}/${item.longitude}`);
    } else {
      history.push(`/map/lot/${item.lotid}`);
    }
  };

  const searchWithTerms = () => {
    if (searchState.length === 0) {
      history.push("/map/nearby");
    } else if (currentPos != null) history.push(`/map/search/${searchState}/${currentPos.lat}/${currentPos.lng}`);
    else history.push(`/map/search/${searchState}`);
  };

  return (
    <div className={panelDisplay === "full" ? "flex w-full mainbar-container z-[1100] py-2" : "absolute w-screen py-2 mainbar-container z-[1100]"}>
      <div
        className={
          panelDisplay === "full" ? "flex items-center px-2 py-1 bg-white border w-[95%] max-w-screen-sm mx-auto" : "rounded-full flex items-center px-2 py-1 bg-white border w-[95%] max-w-screen-sm mx-auto"
        }
      >
        <div className="flex-none">
          <button onClick={handleMenuToggled} className="px-3 py-2 rounded text-slate-700">
            <i className="fa-solid fa-bars"></i>
          </button>
        </div>
        <div className="relative flex-col flex-1 border-l-2 border-red-500">
          <form
            onSubmit={(e) => {
              e.preventDefault();
              if (searchState.length === 0) return;
              if (currentPos != null) history.push(`/map/search/${searchState}/${currentPos.lat}/${currentPos.lng}`);
              else history.push(`/map/search/${searchState}`);
              setSearchState("");
            }}
          >
            <input
              className="flex w-full px-3 text-sm focus:outline-none text-slate-600"
              type="search"
              name="search"
              id="query"
              autoComplete="off"
              onBlur={onBlur}
              onFocus={onFocus}
              onChange={(e) => setSearchState(e.target.value)}
              placeholder="Find buildings, lots..."
              defaultValue={terms ? terms : ""}
            ></input>
            {/* Map results */}
            <div className="absolute flex flex-col w-full bg-white shadow-2xl rounded-b-md">
              {searchResults && focused &&
                searchResults.map((item) => {
                  function getHighlightedText(text) {
                    if (!searchState || searchState.length === 0 || !text)
                      return <span>{text}</span>
                    // Split on highlight term and include term into parts, ignore case
                    let highlight = searchState;
                    const parts = text.toString().split(new RegExp(`(${highlight})`, "gi"));
                    return (
                      <span>
                        {" "}
                        {parts.map((part, i) => (
                          <span
                            key={i}
                            className={part.toLowerCase() === highlight.toLowerCase() ? "font-semibold text-slate-800" : ""}
                          >
                            {part}
                          </span>
                        ))}{" "}
                      </span>
                    );
                  }

                  return (
                    <div
                      key={item.isbuilding ? `building${item.name}`:item.lotid}
                      onClick={(e)=>{
                        handleOnSelect(item);
                      }}
                      onMouseDown={()=>{
                        handleOnSelect(item);
                      }}
                      className="flex flex-row justify-start p-2 text-sm flex-nowrap hover:text-slate-500 hover:bg-slate-300/60">
                      {item.isbuilding ? (
                        <span className="truncate">
                          <i class={"fas fa-building text-slate-500"}></i>
                          <span className={"mr-1 text-[9px] tracking-tighter text-slate-400 align-sub "}>nearby</span>
                          {getHighlightedText(`${item.shortname} • ` + item.name)}
                        </span>
                      ) : (
                        <span className="truncate">
                          <i class={"fas fa-square-parking text-indigo-500 text-lg"}></i>
                          {getHighlightedText(item.name)}
                        </span>
                      )}
                    </div>
                  );
                })}
            </div>
          </form>
        </div>
        <div className="flex-none">
          <button className="px-3 py-2 rounded" onClick={searchWithTerms} disabled={searchState === ""}>
            <i
              className={"fa-solid fa-magnifying-glass " + (searchState === "" ? "text-slate-400" : "text-slate-900")}
            ></i>
          </button>
        </div>
      </div>
    </div>
  );
}
export default MainBar;
