import axios from "axios";
import React, { useState, Component, useEffect, Fragment } from "react";
import { useSelector, useDispatch } from "react-redux";
import { showMenuBar, hideMenuBar } from "../../redux/menuSlice";
import { Dialog, Transition } from '@headlessui/react'; // For announcement dialog box
import LegendToggle from "./map/LegendToggle.js"
import FilterMenu from "./filters/FilterMenu";

function UserMenu() {
  const menuToggled = useSelector((state) => state.menuToggle.toggled);
  const dispatch = useDispatch();
  const [announcement, setAnnouncement] = useState();
  const [apiError, setApiError] = useState();
  const [modalShowing, setModalShowing] = useState(false);

  const handleMenuToggled = (e) => {
    e.preventDefault();
    if (menuToggled) dispatch(hideMenuBar());
    else dispatch(showMenuBar());
  };

  // Open and close announcement dialog box
  function closeModal() {
    setModalShowing(false);
  }
  function openModal() {
    setModalShowing(true);
  }

  useEffect(() => {
    // Get Admin Announcement for Systemwide announcement (lot 0)
    axios({
      url: `${process.env.REACT_APP_API_URL}/api/adminannouncement/${0}/`,
      method: "GET",
      headers: {
        "Content-Type": "application/json",
      },
    })
      .then((response) => {
        setAnnouncement(response.data);
      })
      .catch((error) => {
        setApiError(error);
      });
  }, []);

  // Find current date for expiration calculation
  const currentDate = new Date();
  currentDate.setHours(0,0,0,0);

  return (
    <div className="z-[1500]">
      <div
        className="rounded-tr-xl rounded-br-lg mobile-menu sidebar z-[1500] bg-white text-dark-700 border-2 shadow-lg w-72 space-y-10 py-7 px-2 absolute inset-y-0 left-0 transform transition duration-200 ease-in-out"
        style={{
          transform: menuToggled ? "translateX(0)" : "translateX(-100%)"
        }}
      >
        <div className="z-[1500]">

          <button onClick={()=>{dispatch(hideMenuBar())}} className="block w-full px-2 py-2 mb-2 text-sm font-semibold text-white text-amaranth-400">
            <i className='self-center w-8 h-8 text-lg border rounded-full shadow-lg border-amaranth-300/20 fa-solid fa-chevron-left'/>
          </button>
          <LegendToggle/>
          <FilterMenu/>
          {announcement &&
          <div className="flex flex-row flex-nowrap w-[100%]"
              style={{
                display: (announcement.announcement === "N/A" || (new Date(announcement.startDate) > currentDate && currentDate < new Date(announcement.endDate))) ? "none":"flex"
              }}>
                        <div onClick={openModal} className="flex flex-row px-1 mx-2 mb-1 truncate border rounded border-yellow-700/60 bg-yellow-400/40 place-self-center place-items-center">
                        <i className="text-lg text-yellow-500 -translate-y-px fa-solid fa-triangle-exclamation"></i>
                        <span  className="px-2 truncate yellow-800">
                          {announcement.announcement}
                        </span>
                        {/* Announcement dialog box*/}
                        <Transition appear show={modalShowing} as={Fragment}>
                            <Dialog
                              as="div"
                              className="fixed inset-0 z-[1500] overflow-y-auto"
                              onClose={closeModal}
                            >
                              <div className="min-h-screen px-4 text-center">
                                <Transition.Child
                                  as={Fragment}
                                  enter="ease-out duration-300"
                                  enterFrom="opacity-0"
                                  enterTo="opacity-100"
                                  leave="ease-in duration-200"
                                  leaveFrom="opacity-100"
                                  leaveTo="opacity-0"
                                >
                                   <Dialog.Overlay className="fixed inset-0 bg-black opacity-30" />
                                </Transition.Child>

                                {/* This element is to trick the browser into centering the modal contents. */}
                                <span
                                  className="inline-block h-screen align-middle"
                                  aria-hidden="true"
                                >
                                  &#8203;
                                </span>
                                <Transition.Child
                                  as={Fragment}
                                  enter="ease-out duration-300"
                                  enterFrom="opacity-0 scale-95"
                                  enterTo="opacity-100 scale-100"
                                  leave="ease-in duration-200"
                                  leaveFrom="opacity-100 scale-100"
                                  leaveTo="opacity-0 scale-95"
                                >
                                  <div className="inline-block w-full max-w-md p-6 my-8 overflow-hidden text-left align-middle transition-all transform bg-white shadow-xl rounded-2xl">
                                    <Dialog.Title
                                      as="h3"
                                      className="mx-2 text-lg font-medium leading-6 text-center text-gray-900"
                                    >
                        <i className="mr-2 text-xl text-yellow-500 fa-solid fa-triangle-exclamation"></i>

                                      Systemwide Announcement
                                    </Dialog.Title>
                                    <div className="p-2 mt-2">
                                      <p className="text-sm text-slate-800">
                                        {announcement.announcement}
                                      </p>
                                    </div>

                                    <div className="mt-4">
                                      <button
                                        type="button"
                                        className="inline-flex justify-center px-4 py-2 text-sm font-medium bg-yellow-100 border border-transparent rounded-md text-yellow-900/80 hover:bg-red-200 focus:outline-none focus-visible:ring-2 focus-visible:ring-offset-2 focus-visible:ring-rose-500"
                                        onClick={closeModal}
                                      >
                                        Got it, thanks!
                                      </button>
                                    </div>
                                  </div>
                                </Transition.Child>
                              </div>
                            </Dialog>
                          </Transition>
                        </div>
            </div>}
        </div>
      </div>
      <button
        onClick={handleMenuToggled}
        className="absolute block translate-x-0 z-[1200] inset-y-0 left-0 bg-gray-700 bg-opacity-50 h-full w-full"
        style={{ display:
          menuToggled ? "block" : "none"
        }}
      ></button>
    </div>
  );
}

export default UserMenu;