import React, { useState, useEffect } from "react";
import { GeoJSON, useMap } from "react-leaflet";
import { useHistory, useLocation, matchPath } from "react-router";
import { useDispatch, useSelector } from "react-redux";
import { showHalf, showFull, hideLotSelect, showComponent } from "../../../redux/lotSelectSlice";
import { gray } from "d3";

const useToSelectedLotID = () => {
  const currentURL = useLocation();
    const match = matchPath(currentURL.pathname, {
      path: "/map/lot/:id",
      exact: true,
    });   

    return match?.params?.id  
}

export default function LotsLayer(props) {
  const lotStallsRemaining = useSelector((state) => state.map.lotStallsRemaining);
  // a mapping from lot id's to a list of their accepted passes
  const lotPasses = useSelector((state) => state.map.lotPasses);
  // a list of dictionaries containing name/shortname of all passes
  const Toggles = useSelector((state) => state.filterToggles.Toggles);
  const [alreadyPanToMap, setAlreadyPanToMap] = useState(false);
  const [randomKey, setRandomKey] = useState(0);
  const dispatch = useDispatch();
  const selectedLotID = useToSelectedLotID();

  const geoJsonRef = React.useRef();
  let history = useHistory();
  const map = useMap();

  useEffect(() => {
    setRandomKey(state => state + 1);
  }, [Toggles, selectedLotID])

  useEffect(() => {
    if (map && geoJsonRef.current && !alreadyPanToMap) {

      setTimeout(() => {
        map.flyToBounds(geoJsonRef.current.getBounds());
        setAlreadyPanToMap(true);
      }, 500);
    }
  });
  const userMapFlyToLatLng = useSelector((state) => state.map.userMapFlyToLatLng);
  const userMapFlyToZoomLevel = useSelector((state) => state.map.userMapFlyToZoomLevel);
  // UseEffect to fly to bounds when userMapFlyToLatLng changes
  useEffect(()=>{
    if (map && geoJsonRef.current && userMapFlyToLatLng){
      if (userMapFlyToLatLng)
        map.setView(userMapFlyToLatLng, userMapFlyToZoomLevel);
      else
        map.setView(userMapFlyToLatLng);
    }
    //reset map
    if (map && geoJsonRef.current && alreadyPanToMap &&!userMapFlyToLatLng) {
      map.flyToBounds(geoJsonRef.current.getBounds());
  }
  },[userMapFlyToLatLng]);

  // This function colors lots based on total spaces still available. It is the current implementation
  function setColorThresholdRemainingStalls(stallsRemaining, lotid,selectedLotID) {

    const gray = "#777777";
    const green = "#60d394";
    const yellow = "#ffc55d";
    const red = "#ee6055";
    const error_color = "#d3d3d3";

    // Turn every lot but the one selected gray
    // if (selectedLotID && selectedLotID != lotid)
    // {     
    //   return gray;     
    // }

    // Check pass filters
    if (lotid in lotPasses) {
      const lot_passes = lotPasses[lotid].slice();

      var toggled_passes = [];
      for (const [key, val] of Object.entries(Toggles)){
        if (val){
          toggled_passes.push(key);
        }
      }

      const in_toggled = (elem) => toggled_passes.includes(elem);

      if (!lot_passes.some(in_toggled)) {
        return gray;
      }
    }

    // No used stall data exists color
    if (stallsRemaining === "nodata") {
      return error_color;
    }
    // most available stalls
    if (stallsRemaining > 50) {
      return green;
    }
    // medium available stalls
    else if (stallsRemaining > 25) {
      return yellow;
    }
    // least available stalls
    else {
      return red;
    }
  }

  // Set the opacity of each geoJSON polygon
  function setOpacity(lotid,selectedLotID) {

    // If there is a selection
    if (selectedLotID)
    {
      // Set opacity for the selected lot
      if (selectedLotID == lotid)
      {
        return {opacity: 1, fillOpacity: 0.8}
      }
      // Set opacity for every other lot not selected
      else
      {
        return {opacity: 0.5, fillOpacity: 0.2}
      }
    }

    // Fill everything the same when nothing is selected
    return {opacity: 1, fillOpacity: 0.5}

  }

  //DO WHATEVER YOU WANT WITH EACH feature/layer
  // Note: lotStallsRemaining changes to lotPercents if a percentage threshold is used
  const onEachFeature = React.useCallback((feature, layer) => {
    const currentLotID = parseInt(feature.properties.PLOT);
    layer.setStyle({
      color: setColorThresholdRemainingStalls(lotStallsRemaining[currentLotID], currentLotID,selectedLotID),
      //color: "unset",
      weight:1,
      ...setOpacity(currentLotID,selectedLotID),
    });

    const onClick = (e) => {
      e.originalEvent.view.L.DomEvent.stopPropagation(e)
      setRandomKey(state => state + 1);
      history.push(`/map/lot/${parseInt(feature.properties.PLOT)}`)};

    //Bind click:
    layer.on({ click: onClick })

  }, [lotStallsRemaining, lotPasses, Toggles,selectedLotID]);

  const Layer = <GeoJSON key={randomKey} ref={geoJsonRef} data={props.mapData} onEachFeature={onEachFeature}></GeoJSON>;

  return (
    <>
      {/* Only render Geojson when mapData (geojson) and percents are loaded */}
      {props.mapData && Object.keys(lotStallsRemaining).length === Object.keys(props.mapData.features).length && Layer}
    </>
  );
}
